import os
import envi
import vivisect.exc as v_exc
import vivisect.parsers as v_parsers
from vivisect.const import *


archcalls = {
    'i386': 'cdecl',
    'amd64': 'sysvamd64call',
    'arm': 'armcall',
    'thumb': 'armcall',
    'thumb16': 'armcall',
}


def parseFd(vw, fd, filename=None, baseaddr=None):
    fd.seek(0)
    arch = vw.config.viv.parsers.blob.arch
    bigend = vw.config.viv.parsers.blob.bigend
    if baseaddr is None:
        baseaddr = vw.config.viv.parsers.blob.baseaddr
    try:
        envi.getArchModule(arch)
    except Exception:
        raise v_exc.BlobArchException()

    if filename is None:
        filename = 'blob_%.8x' % baseaddr

    vw.setMeta('Architecture', arch)
    vw.setMeta('Platform', 'unknown')
    vw.setMeta('Format', 'blob')

    vw.setMeta('bigend', bigend)
    vw.setMeta('DefaultCall', archcalls.get(arch, 'unknown'))

    bytez = fd.read()
    fname = vw.addFile(filename, baseaddr, v_parsers.md5Bytes(bytez))
    vw.addMemoryMap(baseaddr, 7, fname, bytez)
    vw.addSegment(baseaddr, len(bytez), '%.8x' % baseaddr, fname)
    vw.setFileMeta(fname, 'sha256', v_parsers.sha256Bytes(bytez))

def parseFile(vw, filename, baseaddr=None):

    arch = vw.config.viv.parsers.blob.arch
    bigend = vw.config.viv.parsers.blob.bigend
    if baseaddr is None:
        baseaddr = vw.config.viv.parsers.blob.baseaddr

    try:
        envi.getArchModule(arch)
    except Exception as e:
        raise v_exc.BlobArchException() from None

    vw.setMeta('Architecture', arch)
    vw.setMeta('Platform', 'unknown')
    vw.setMeta('Format', 'blob')

    vw.setMeta('bigend', bigend)
    vw.setMeta('DefaultCall', archcalls.get(arch, 'unknown'))

    with open(filename, 'rb') as f:
        bytez = f.read()
    fname = vw.addFile(filename, baseaddr, v_parsers.md5File(filename))
    vw.setFileMeta(fname, 'sha256', v_parsers.sha256Bytes(bytez))
    vw.addMemoryMap(baseaddr, 7, fname, bytez)
    vw.addSegment(baseaddr, len(bytez), '%.8x' % baseaddr, fname)


def parseMemory(vw, memobj, baseaddr):
    va, size, perms, fname = memobj.getMemoryMap(baseaddr)
    if not fname:
        fname = 'map_%.8x' % baseaddr

    bytez = memobj.readMemory(va, size)
    fname = vw.addFile(fname, baseaddr, v_parsers.md5Bytes(bytez))
    vw.setFileMeta(fname, 'sha256', v_parsers.sha256Bytes(bytez))
    vw.addMemoryMap(va, perms, fname, bytez)
    vw.setMeta('DefaultCall', archcalls.get(arch,'unknown'))

def getMemBaseAndSize(vw, filename, baseaddr=None):
    if baseaddr is None:
        baseaddr = vw.config.viv.parsers.blob.baseaddr
    size = os.lstat(filename).st_size
    return baseaddr, size




import sys
import argparse
import traceback

import vstruct.defs.java as v_d_java


def setup():
    ap = argparse.ArgumentParser(description='Java Class File Printer')
    ap.add_argument('file', nargs='+', help='A bunch of java class files to parse')
    return ap


def main(argv):
    opts = setup().parse_args(argv)

    for filename in opts.file:
        with open(filename, 'rb') as f:
            javacls = v_d_java.JavaClass()
            try:
                javacls.vsParse(f.read())
                print(javacls.tree())

                cname = javacls.getClassName()
                sname = javacls.getSuperClassName()
                print('Java Class: %s (inherits: %s)' % (cname, sname))

                for fname, descname, attrs in javacls.getClassFields():
                    print('Field: %s (%s) (attrs: %r)' % (fname, descname, attrs.keys()))

                for methname, attrs in javacls.getClassMethods():
                    print('Method: %s (attrs: %r)' % (methname, attrs.keys()))

                print('Constants:')
                for fname, const in javacls.const_pool:
                    print(const.tag, const.data.tree())

                print(javacls.getClassAttributes().keys())

            except Exception:
                print(javacls.tree())
                traceback.print_exc()


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import logging

import envi.archs.amd64 as e_amd64
import vivisect.impemu.emulator as v_i_emulator

logger = logging.getLogger(__name__)
########################################################################
#
# NOTE: For each architecture we intend to do workspace emulation on,
#       extend an emulator to allow any of the needed tweaks (rep prefix
#       etc).
# NOTE: ARCH UPDATE

logger = logging.getLogger(__name__)
non_use_mnems = ('push', )


class Amd64WorkspaceEmulator(v_i_emulator.WorkspaceEmulator, e_amd64.Amd64Emulator):
    __archemu__ = e_amd64.Amd64Emulator

    taintregs = [
        e_amd64.REG_RAX, e_amd64.REG_RCX, e_amd64.REG_RDX,
        e_amd64.REG_RBX, e_amd64.REG_RBP, e_amd64.REG_RSI,
        e_amd64.REG_RDI, e_amd64.REG_R8,  e_amd64.REG_R9,
    ]

    def __init__(self, vw, **kwargs):
        '''
        Please see the base emulator class in vivisect/impemu/emulator.py for the parameters
        that can be passed through kwargs
        '''
        e_amd64.Amd64Emulator.__init__(self)
        v_i_emulator.WorkspaceEmulator.__init__(self, vw, **kwargs)
        self.setEmuOpt('i386:repmax', 1)

    def getRegister(self, index):
        """
        Return the current value of the specified register index.
        """
        value = e_amd64.Amd64Emulator.getRegister(self, index)

        if self.op is None:
            return value

        # this is broken, but works enough to keep for now.  if we
        # run into new 64-bit calling conventions, we may need to fix
        # this.
        if index not in self.taintregs:
            return value

        if self.isRegUse(self.op, index):
            self._useVirtAddr(value)
        return value

    def isRegUse(self, op, ridx):
        '''
        determines if the sequence of uses(get/setRegister) is a register 'use'.
        '''
        # conditions that indicate 'usage':
        # * 'normal usage'
        #   read from a register with *no* previous writes/sets to that register

        # conditions that do *not* indicate 'usage':
        # * 'register initialization', ie xor rax, rax, mov rax, 0, etc
        # * 'save/restore of register', ie push/pop

        # blacklist of mnems that do *not* indicate a 'use'

        # if first item is a set, then this is *not* an arg(register) use
        # (reg initialization)
        if op.mnem == 'xor' and op.opers[0] == op.opers[1]:
            # xor register initialization
            return False

        else:
            # if op mnem is in blacklist, it's not a use either
            for nonuse_mnem in non_use_mnems:
                if nonuse_mnem in repr(op):
                    return False

        return True



import string

from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import *

import envi.memory as e_mem
import envi.const as e_const
import envi.memcanvas as e_canvas
import envi.memcanvas.renderers as e_render
from vqt.main import getSaveFileName

class MemSearchDialog(QDialog):
    '''
    gui for search cli command.
    '''
    def __init__(self):
        QDialog.__init__(self)

        self.modes = ['ascii', 'hex', 'regex', 'utf-8', 'utf-16-le', 'utf-16-be']
        self.pattern = None
        self.filename = None

        rend = e_render.ByteRend()
        self.canvas = e_canvas.StringMemoryCanvas(None)
        self.canvas.addRenderer('bytes', rend)

        hbox1 = QHBoxLayout()
        mode_label = QLabel('Input: ')
        self.mode_combo = QComboBox()
        self.mode_combo.addItems(self.modes)
        self.mode_combo.currentIndexChanged.connect(self.encodingChanged)
        hbox1.addWidget(mode_label)
        hbox1.addWidget(self.mode_combo, alignment=QtCore.Qt.AlignLeft)
        hbox1.addStretch(1)

        hbox2 = QHBoxLayout()
        data_label = QLabel('Bytes: ')
        self.data_edit = QLineEdit()
        hbox2.addWidget(data_label)
        hbox2.addWidget(self.data_edit)

        vbox1 = QVBoxLayout()
        vbox1.addLayout(hbox1)
        vbox1.addLayout(hbox2)

        gbox1 = QGroupBox('Search Criteria')
        gbox1.setLayout(vbox1)

        hbox3 = QHBoxLayout()
        vbox_hex_label = QVBoxLayout() # for align to top.
        hex_label = QLabel('Hex:   ')
        vbox_hex_label.addWidget(hex_label, alignment=QtCore.Qt.AlignTop)
        self.hex_edit = QPlainTextEdit()
        self.hex_edit.setReadOnly(True)
        font = QtGui.QFont('Courier') # should use actual memcanvas.
        self.hex_edit.setFont(font)
        hbox3.addLayout(vbox_hex_label)
        hbox3.addWidget(self.hex_edit)

        vbox2 = QVBoxLayout()
        vbox2.addLayout(hbox3)

        gbox2 = QGroupBox('Bytes to Search For')
        gbox2.setLayout(vbox2)

        hbox4 = QHBoxLayout()
        save_check = QCheckBox('Save Search Results')
        save_check.stateChanged.connect(self.checkChanged)
        self.fname_label = QLabel('')
        buttons = QDialogButtonBox()
        buttons.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        buttons.accepted.connect(self.okClicked)
        buttons.rejected.connect(self.cancelClicked)
        hbox4.addWidget(save_check)
        hbox4.addWidget(self.fname_label)
        hbox4.addWidget(buttons)

        vbox = QVBoxLayout()
        vbox.addWidget(gbox1)
        vbox.addWidget(gbox2)
        vbox.addLayout(hbox4)

        self.setLayout(vbox)

        self.setWindowTitle('Memory Search')
        self.resize(650, 300)
        self.data_edit.setFocus()

    def keyReleaseEvent(self, event):
        encoding = self.mode_combo.currentText()
        self.encodingChanged(None)

    def checkChanged(self, state):
        if state == QtCore.Qt.Checked:
            self.showSaveAsDialog()
        else:
            self.fname_label.setText('')

    def encodingChanged(self, idx):
        encoding = str(self.mode_combo.currentText())

        validator = None
        if encoding == 'hex':
            # only clear the box if there are non-hex chars
            # before setting the validator.
            txt = str(self.data_edit.text())
            if not all(c in string.hexdigits for c in txt):
                self.data_edit.setText('')

            regex = QtCore.QRegExp('^[0-9A-Fa-f]+$')
            validator = QtGui.QRegExpValidator(regex)

        self.data_edit.setValidator(validator)

        txt = str(self.data_edit.text())
        txt_encoded = self.encodeData(txt, encoding)
        self.updateHexPreview(txt_encoded)

    def encodeData(self, txt, encoding):
        if encoding == 'hex' and (len(txt) % 2) != 0:
            txt = txt[:-1] # trim last if odd length

        if encoding == 'hex':
            if not all(c in string.hexdigits for c in txt):
                return None

            return txt.decode(encoding)

        elif encoding == 'regex':
            return None

        return txt.encode(encoding)

    def updateHexPreview(self, bytez):
        if bytez is None:
            self.hex_edit.setPlainText('')
            return

        self.canvas.clearCanvas()
        mem = e_mem.MemoryObject()
        mem.addMemoryMap(0, e_const.MM_READ, b'', bytez)
        self.canvas.mem = mem
        self.canvas.renderMemory(0, len(bytez))
        self.hex_edit.setPlainText(str(self.canvas))

    def showSaveAsDialog(self):
        fname = str(getSaveFileName(caption='Select file to save results to'))
        self.fname_label.setText(fname)

    def cancelClicked(self):
        self.close()

    def okClicked(self):
        self.pattern = (self.data_edit.text())
        self.filename = str(self.fname_label.text())

        self.accept()
        self.close()

    def getResults(self):
        return self.pattern, self.filename



import logging

import vstruct
from vstruct.primitives import *

logger = logging.getLogger(__name__)

class MiniDumpString(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Length = v_uint32()

    def pcb_Length(self):
        self.Buffer = v_zwstr()

class VS_FixedFileInfo(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Signature = v_uint32()
        self.StrucVersion = v_uint32()
        self.FileVersionMS = v_uint32()
        self.FileVersionLS = v_uint32()
        self.ProductVersionMS = v_uint32()
        self.ProductVersionLS = v_uint32()
        self.FileFlagsMask = v_uint32()
        self.FileFlags = v_uint32()
        self.FileOS = v_uint32()
        self.FileType = v_uint32()
        self.FileSubType = v_uint32()
        self.FileDateMS = v_uint32()
        self.FileDateLS = v_uint32()

class MiniDumpLocationDescriptor(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.DataSize = v_uint32()
        self.RVA = v_uint32()

class MiniDumpMemoryDescriptor(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.StartOfMemoryRange = v_uint64()
        self.Memory = MiniDumpLocationDescriptor()

class MiniDumpMemoryDescriptor64(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.StartOfMemoryRange = v_uint64()
        self.DataSize = v_uint64()

class MiniDumpLastReservedStream(vstruct.VStruct):
    '''
    Stream Type: 0xffff
        LastReservedStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)

class MiniDumpJavaScriptDataStream(vstruct.VStruct):
    '''
    Stream Type 20:
        JavaScriptDataStream
        Defined in DbgHelp.h _MINIDUMP_STREAM_TYPE struct, no other reference
    '''
    # TODO: not implemented
    def __init__(self):
        vstruct.VStruct.__init__(self)

class MiniDumpTokenInfoHeader(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.TokenSize = v_uint32()
        self.TokenId = v_uint32()
        self.TokenHandle = v_uint64()

    def pcb_TokenHandle(self):
        # inclusive length counter, subtract len of header size
        if self.TokenSize < len(MiniDumpTokenInfoHeader()):
            raise Exception('unexpected token size')

        self.Token = v_bytes(self.TokenSize - len(MiniDumpTokenInfoHeader()))

class MiniDumpTokenInfoListStream(vstruct.VStruct):
    '''
    Stream Type 19:
        TokenStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.TokenListSize = v_uint32()
        self.TokenListEntries = v_uint32()
        self.ListHeaderSize = v_uint32()
        self.ElementHeaderSize = v_uint32()

    def pcb_ElementHeaderSize(self):
        self.Entries = vstruct.VArray([MiniDumpTokenInfoHeader() for i in range(self.TokenListEntries)])

AVRF_MAX_TRACES = 32

class AvrfBacktraceInformation(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Depth = v_uint32()
        self.Index = v_uint32()
        self.ReturnAddresses = vstruct.VArray([v_uint64() for i in range(AVRF_MAX_TRACES)])

class AvrfHandleOperationList(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Handle = v_uint64()
        self.ProcessId = v_uint32()
        self.ThreadId = v_uint32()
        self.OperationType = v_uint32()
        self.Spare0 = v_uint32()
        self.BackTraceInformation = AvrfBacktraceInformation()

class MiniDumpHandleOperationListStream(vstruct.VStruct):
    '''
    Stream Type 18:
        HandleOperationListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfHeader = v_uint32()
        self.SizeOfEntry = v_uint32()
        self.NumberOfEntries = v_uint32()
        self.Reserved = v_uint32()

    def pcb_Reserved(self):
        self.Entries = vstruct.VArray([AvrfHandleOperationList() for i in range(self.NumberOfEntries)])

class MiniDumpThreadInfo(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.ThreadId = v_uint32()
        self.DumpFlags = v_uint32()
        self.DumpErro = v_uint32()
        self.ExitStatus = v_uint32()
        self.CreateTime = v_uint64()
        self.ExitTime = v_uint64()
        self.KernelTime = v_uint64()
        self.UserTime = v_uint64()
        self.StartAddress = v_uint64()
        self.Affinity = v_uint64()

class MiniDumpThreadInfoListStream(vstruct.VStruct):
    '''
    Stream Type 17:
        ThreadInfoListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfHeader = v_uint32()
        self.SizeOfEntry = v_uint32()
        self.NumberOfEntries = v_uint32()

    def pcb_NumberOfEntries(self):
        self.ThreadInfo = vstruct.VArray([MiniDumpThreadInfo() for i in range(self.NumberOfEntries)])

class MiniDumpMemoryInfo(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.BaseAddress = v_uint64()
        self.AllocationBase = v_uint64()
        self.AllocationProtect = v_uint32()
        self.__alignment = v_uint32()
        self.RegionSize = v_uint64()
        self.State = v_uint32()
        self.Protect = v_uint32()
        self.Type = v_uint32()
        self.__alignment2 = v_uint32()

class MiniDumpMemoryInfoListStream(vstruct.VStruct):
    '''
    Stream Type 16:
        MemoryInfoListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfHeader       = v_uint32()
        self.SizeOfEntry        = v_uint32()
        self.NumberOfEntries    = v_uint64()

    def pcb_NumberOfEntries(self):
        self.Entries            = vstruct.VArray([MiniDumpMemoryInfo() for i in range(self.NumberOfEntries)])

class MiniDumpMiscInfo2Stream(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfInfo = v_uint32()
        self.Flags1 = v_uint32()
        self.ProcessId = v_uint32()
        self.ProcessCreateTime = v_uint32()
        self.ProcessUserTime = v_uint32()
        self.ProcessKernelTime = v_uint32()
        self.ProcessorMaxMhz = v_uint32()
        self.ProcessorCurrentMhz = v_uint32()
        self.ProcessorMhzLimit = v_uint32()
        self.ProcessorMaxIdleState = v_uint32()
        self.ProcessorCurrentIdleState = v_uint32()

class MiniDumpMiscInfoStream(vstruct.VStruct):
    '''
    Stream Type 15:
        MiscInfoStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfInfo = v_uint32()
        self.Flags1 = v_uint32()
        self.ProcessId = v_uint32()
        self.ProcessCreateTime = v_uint32()
        self.ProcessUserTime = v_uint32()
        self.ProcessKernelTime = v_uint32()

class MiniDumpUnloadedModule(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.BaseOfImage = v_uint64()
        self.SizeOfImage = v_uint32()
        self.CheckSum = v_uint32()
        self.TimeDateStamp = v_uint32()
        # TODO: RVA to MINIDUMP_STRING
        self.ModuleNameRVA = v_uint32()

class MiniDumpUnloadedModuleListStream(vstruct.VStruct):
    '''
    Stream Type 14:
        UnloadedModuleListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfHeader = v_uint32()
        self.SizeOfEntry = v_uint32()
        self.NumberOfEntries = v_uint32()

    def pcb_NumberOfEntries(self):
        self.Entries = vstruct.VArray([MiniDumpUnloadedModule() for i in range(self.NumberOfEntries)])

class MiniDumpFunctionTableDescriptor(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.MinimumAddress = v_uint64()
        self.MaximumAddress = v_uint64()
        self.BaseAddress = v_uint64()
        self.EntryCount = v_uint32()
        self.SizeOfAlignPad = v_uint32()

    # TODO: unfinished

class MiniDumpFunctionTableStream(vstruct.VStruct):
    '''
    Stream Type 13:
        FunctionTableStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfHeader = v_uint32()
        self.SizeOfDescriptor = v_uint32()
        self.SizeOfNativeDescriptor = v_uint32()
        self.SizeOfFunctionEntry = v_uint32()
        self.NumberOfDescriptors = v_uint32()
        self.SizeOfAlignPad = v_uint32()

    def pcb_SizeOfAlignPad(self):
        self.Padding = v_bytes(self.SizeOfAlignPad)

class MiniDumpHandleObjectInformation(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.NextInfoRva = v_uint32()
        self.InfoType = v_uint32()
        self.SizeOfInfo = v_uint32()

    def pcb_SizeOfInfo(self):
        self.Information = v_bytes(self.SizeOfInfo)

class MiniDumpHandleDescriptor2(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Handle = v_uint64()
        # TODO: RVA to MINIDUMP_STRING
        self.TypeNameRva = v_uint32()
        # TODO: RVA to MINIDUMP_STRING
        self.ObjectNameRva = v_uint32()
        self.Attributes = v_uint32()
        self.GrantedAccess = v_uint32()
        self.HandleCount = v_uint32()
        self.PointerCount = v_uint32()
        # TODO: RVA to MINIDUMP_HANDLE_OBJECT_INFORMATION
        self.ObjectInfoRva = v_uint32()
        self.Reserved0 = v_uint32()

class MiniDumpHandleDescriptor(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Handle = v_uint32()
        # TODO: RVA to MINIDUMP_STRING
        self.TypeNameRva = v_uint32()
        # TODO: RVA to MINIDUMP_STRING
        self.ObjectNameRva = v_uint32()
        self.Attributes = v_uint32()
        self.GrantedAccess = v_uint32()
        self.HandleCount = v_uint32()
        self.PointerCount = v_uint32()

class MiniDumpHandleDataStream(vstruct.VStruct):
    '''
    Stream Type 12:
        HandleDataStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.SizeOfHeader = v_uint32()
        self.SizeOfDescriptor = v_uint32()
        self.NumberOfDescriptors = v_uint32()
        self.Reserved = v_uint32()

    def pcb_Reserved(self):
        if self.SizeOfDescriptor == 32:     # Handle Descriptor
            self.Descriptors = vstruct.VArray([MiniDumpHandleDescriptor() for i in range(self.NumberOfDescriptors)])
        elif self.SizeOfDescriptor == 40:   # Handle Descriptor 2
            self.Descriptors = vstruct.VArray([MiniDumpHandleDescriptor2() for i in range(self.NumberOfDescriptors)])
        else:                               # Handle Descriptor Unknown
            raise Exception('unknown Handle Descriptor version')

class MiniDumpCommentStreamW(vstruct.VStruct):
    '''
    Stream Type 11:
        CommentStreamW
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.CommentW = v_zwstr()

class MiniDumpCommentStreamA(vstruct.VStruct):
    '''
    Stream Type 10:
        CommentStreamA
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.CommentA = v_zstr()

class MiniDumpMemory64ListStream(vstruct.VStruct):
    '''
    Stream Type 9:
        MemoryList64Stream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.NumberOfMemoryRanges = v_uint64()
        self.BaseRva = v_uint64()

    def pcb_BaseRva(self):
        self.MemoryRanges =  vstruct.VArray([MiniDumpMemoryDescriptor64() for i in range(self.NumberOfMemoryRanges)])

class MiniDumpThreadEx(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.ThreadId = v_uint32()
        self.SuspendCount = v_uint32()
        self.PriorityClass = v_uint32()
        self.Priority = v_uint32()
        self.Teb = v_uint64()
        self.Stack = MiniDumpMemoryDescriptor()
        self.ThreadContext = MiniDumpLocationDescriptor()
        self.BackingStore = MiniDumpMemoryDescriptor()

class MiniDumpThreadExListStream(vstruct.VStruct):
    '''
    Stream Type 8:
        ThreadExListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.NumberOfThreads = v_uint32()

    def pcb_NumberOfThread(self):
        self.Threads = vstruct.VArray([MiniDumpThreadEx() for i in range(self.NumberOfThreads)])

class MiniDumpSystemInfoStream(vstruct.VStruct):
    '''
    Stream Type 7:
        SystemInfoStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.ProcessorArchitecture = v_uint16()
        self.ProcessorLevel = v_uint16()
        self.ProcessorRevision = v_uint16()
        self.Reserved = v_uint16()
        self.MajorVersion = v_uint32()
        self.MinorVersion = v_uint32()
        self.BuildNumber = v_uint32()
        self.PlatformId = v_uint32()
        self.CSDVersionRSA = v_uint32()
        self.Reserved1 = v_uint32()
        self.CpuInfo1 = v_uint32()
        self.CpuInfo2 = v_uint32()
        self.CpuInfo3 = v_uint32()
        self.CpuInfo4 = v_uint32()
        self.CpuInfo5 = v_uint32()
        self.CpuInfo6 = v_uint32()

EXCEPTION_MAXIMUM_PARAMETERS = 15

class MiniDumpException(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.ExceptionCode = v_uint32()
        self.ExceptionFlags = v_uint32()
        self.ExceptionRecord = v_uint64()
        self.ExceptionAddress = v_uint64()
        self.NumberParameters = v_uint32()
        self.__unusedAlignment = v_uint32()
        self.ExecptionInformation = vstruct.VArray([v_uint64() for i in range(EXCEPTION_MAXIMUM_PARAMETERS)])

class MiniDumpExceptionStream(vstruct.VStruct):
    '''
    Stream Type 6:
        ExceptionStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.ThreadId = v_uint32()
        self.__aligntment = v_uint32()
        self.ExceptionRecored = MiniDumpException()
        self.ThreadContext = MiniDumpLocationDescriptor()

class MiniDumpMemoryListStream(vstruct.VStruct):
    '''
    Stream Type 5
        MemoryListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.NumberOfMemoryRanges = v_uint32()

    def pcb_NumberOfMemoryRanges(self):
        self.MemoryRanges = vstruct.VArray([MiniDumpMemoryDescriptor() for i in range(self.NumberOfMemoryRanges)])

class MiniDumpModule(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.BaseOfImage = v_uint64()
        self.SizeOfImage = v_uint32()
        self.CheckSum = v_uint32()
        self.TimeDateStamp = v_uint32()
        # TODO: RVA to MINIDUMP_STRING
        self.ModuleNameRva = v_uint32()
        self.VersionInfo = VS_FixedFileInfo()
        self.CvRecord = MiniDumpLocationDescriptor()
        self.MiscRecord = MiniDumpLocationDescriptor()
        self.Reserved1 = v_uint64()
        self.Reserved2 = v_uint64()

class MiniDumpModuleListStream(vstruct.VStruct):
    '''
    Stream Type 4
        ModuleListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.NumberOfModules = v_uint32()

    def pcb_NumberOfModules(self):
        self.Modules = vstruct.VArray([MiniDumpModule() for i in range(self.NumberOfModules)])

class MiniDumpThread(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.ThreadId = v_uint32()
        self.SuspendCount = v_uint32()
        self.PriorityClass = v_uint32()
        self.Priority = v_uint32()
        self.Teb = v_uint64()
        self.Stack = MiniDumpMemoryDescriptor()
        self.ThreadContext = MiniDumpLocationDescriptor()

class MiniDumpThreadListStream(vstruct.VStruct):
    '''
    Stream Type 3:
        ThreadListStream
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.NumberOfThreads = v_uint32()

    def pcb_NumberOfThreads(self):
        self.Threads = vstruct.VArray([MiniDumpThread() for i in range(self.NumberOfThreads)])

class MiniDumpReservedStream1(vstruct.VStruct):
    '''
    Stream Type 2:
        ReservedStream1 - Reserved, do not use this value
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)

class MiniDumpReservedStream0(vstruct.VStruct):
    '''
    Stream Type 1:
        ReservedStream0 - Reserved, do not use this value
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)

class MiniDumpUnusedStream(vstruct.VStruct):
    '''
    Stream Type 0:
        UnusedStream - Reserved, do not use this value
    '''
    def __init__(self):
        vstruct.VStruct.__init__(self)

class MiniDumpDirectory(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.StreamType = v_uint32()
        self.Location = MiniDumpLocationDescriptor()

class MiniDumpHeader(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Signature = v_uint32()
        self.Version = v_uint32()
        self.NumberOfStreams = v_uint32()
        self.StreamDirectoryRVA = v_uint32()
        self.Checksum = v_uint32()
        self.Resv_TimeDateStamp = v_uint32()
        self.Flags = v_uint64()

    def pcb_Flags(self):
        self.Directory = vstruct.VArray([MiniDumpDirectory() for i in range(self.NumberOfStreams)])

streamsDict = { 0: MiniDumpUnusedStream,
                1: MiniDumpReservedStream0,
                2: MiniDumpReservedStream1,
                3: MiniDumpThreadListStream,
                4: MiniDumpModuleListStream,
                5: MiniDumpMemoryListStream,
                6: MiniDumpExceptionStream,
                7: MiniDumpSystemInfoStream,
                8: MiniDumpThreadExListStream,
                9: MiniDumpMemory64ListStream,
                10: MiniDumpCommentStreamA,
                11: MiniDumpCommentStreamW,
                12: MiniDumpHandleDataStream,
                13: MiniDumpFunctionTableStream,
                14: MiniDumpUnloadedModuleListStream,
                15: MiniDumpMiscInfoStream,
                16: MiniDumpMemoryInfoListStream,
                17: MiniDumpThreadInfoListStream,
                18: MiniDumpHandleOperationListStream,
                19: MiniDumpTokenInfoListStream,
                20: MiniDumpJavaScriptDataStream,
                0xFFFF: MiniDumpLastReservedStream,
            }

class MiniDump(object):
    def __init__(self, bytez):
        self.bytez = bytez
        self.header = MiniDumpHeader()
        self.header.vsParse(bytez)

        for idx, header in self.header.Directory:
            # check if streamtype is in our dict of known types
            if header.StreamType in streamsDict:
                sclass = streamsDict[header.StreamType]
                soffset = header.Location.RVA

                # assumption there will be one instance of a stream in minidump
                stream = vars(self)[sclass.__name__] = sclass()
                stream.vsParse(bytez, offset=soffset)
            else:
                logger.info('Unknown stream type of %d', header.StreamType)

    def tree(self):
        txt = []
        txt.append(self.header.tree())
        for idx, header in self.header.Directory:
            # check if streamtype is in our dict of known types
            if header.StreamType in streamsDict:
                sclass = streamsDict[header.StreamType]
                txt.append(vars(self)[sclass.__name__].tree())

        return ''.join(txt)

    def getModuleNameByAddr(self, addr):
        for idx, mod in self.MiniDumpModuleListStream.Modules:
            if addr == mod.BaseOfImage:
                mname = MiniDumpString()
                mname.vsParse(self.bytez, offset=mod.ModuleNameRva)
                return mname.Buffer

    def getMemoryMaps(self):
        maps = []
        # check if stream has been parsed, if not, possibly an incomplete minidump
        if not hasattr(self, 'MiniDumpMemoryInfoListStream'):
            raise Exception('MiniDumpMemoryInfoListStream does not exist. Dump file may not include full memory dump.')

        for idx, entry in self.MiniDumpMemoryInfoListStream.Entries:
            mname =  self.getModuleNameByAddr(entry.AllocationBase)
            if mname is None:
                mname = 'Module Name Not Found'

            maps.append((entry.BaseAddress, entry.RegionSize, entry.Protect, mname))
        return maps

    def readMemory(self, addr, size):
        # check if stream has been parsed, if not, possibly an incomplete minidump
        if not hasattr(self, 'MiniDumpMemory64ListStream'):
            raise Exception('MiniDumpMemory64ListStream does not exist. Dump file may not include full memory dump.')

        offset = self.MiniDumpMemory64ListStream.BaseRva
        for idx, mrange in self.MiniDumpMemory64ListStream.MemoryRanges:
            if addr == mrange.StartOfMemoryRange:
                return self.bytez[offset:offset+size]

            offset += mrange.DataSize

def parseFromBytes(bytez):
    return MiniDump(bytez)

def parseFromFname(fname):
    with open(fname, 'rb') as f:
        bytez = f.read()

    return parseFromBytes(bytez)



"""
A function analysis module that will find code blocks in
functions by flow and xrefs.  This is basically a mandatory
module which should be snapped in *very* early by parsers.

"""
import logging
import collections

import envi
import envi.const as e_const

from vivisect.const import REF_CODE, LOC_POINTER, LOC_OP

logger = logging.getLogger(__name__)

def analyzeFunction(vw, funcva):
    blocks = {}
    done = {}
    mnem = collections.defaultdict(int)
    todo = [funcva, ]
    brefs = []
    size = 0
    opcount = 0
    while len(todo):

        start = todo.pop()

        # If we've hit code we've already done, keep going.
        if done.get(start):
            continue

        done[start] = True
        blocks[start] = 0
        brefs.append((start, True))

        va = start
        op = None
        arch = envi.ARCH_DEFAULT

        # Walk forward through instructions until a branch edge
        while True:
            loc = vw.getLocation(va)

            # If it's not a location, terminate
            if loc is None:
                blocks[start] = va - start
                brefs.append((va, False))
                break

            lva, lsize, ltype, linfo = loc

            if ltype == LOC_POINTER:
                # pointer analysis mis-identified a pointer,
                # so clear and re-analyze instructions.

                vw.delLocation(lva)

                # assume we're adding a valid instruction, which is most likely.
                if op is not None:
                    arch = op.iflags & envi.ARCH_MASK

                vw.makeCode(va, arch=arch, fva=funcva)

                loc = vw.getLocation(va)
                if loc is None:
                    blocks[start] = va - start
                    brefs.append((va, False))
                    break

                lva, lsize, ltype, linfo = loc

            # If it's not an op, terminate
            if ltype != LOC_OP:
                blocks[start] = va - start
                brefs.append((va, False))
                break

            try:
                op = vw.parseOpcode(va)
                mnem[op.mnem] += 1
            except Exception as e:
                logger.warning('Codeblock bad opcode at 0x%x, breaking on error %s', va, e)
                break
            size += lsize
            opcount += 1
            nextva = va + lsize

            # For each of our code xrefs, create a new target.
            branch = False
            xrefs = vw.getXrefsFrom(va, REF_CODE)
            for fromva, tova, rtype, rflags in xrefs:
                # We don't handle procedural branches here...
                if rflags & envi.BR_PROC:
                    continue

                # For now, we'll skip jmp [import] thunks...
                if rflags & envi.BR_DEREF:
                    continue

                mmap = vw.getMemoryMap(tova)
                if mmap:
                    mva, msize, mperm, mname = mmap
                    if mperm & e_const.MM_UNINIT:
                        continue

                branch = True
                todo.append(tova)

            # If it doesn't fall through, terminate (at nextva)
            if linfo & envi.IF_NOFALL:
                blocks[start] = nextva - start
                brefs.append((nextva, False))
                break

            # If we hit a branch, we are the end of a block...
            if branch:
                blocks[start] = nextva - start
                todo.append(nextva)
                break

            if vw.getXrefsTo(nextva, REF_CODE):
                blocks[start] = nextva - start
                todo.append(nextva)
                break

            va = nextva

    oldblocks = {va: size for (va, size, fva) in vw.getFunctionBlocks(funcva)}
    # we now have an ordered list of block references!
    brefs.sort()
    brefs.reverse()
    bcnt = 0
    while len(brefs):
        bva, isbegin = brefs.pop()
        if not isbegin:
            continue

        if len(brefs) == 0:
            break

        # So we don't add a codeblock if we're re-analyzing a function
        # (like during dynamic branch analysis)
        try:
            bsize = blocks[bva]
            if bsize == 0:
                continue

            tmpcb = vw.getCodeBlock(bva)
            # sometimes codeblocks can be deleted if owned by multiple functions
            if bva not in oldblocks or tmpcb is None:
                vw.addCodeBlock(bva, bsize, funcva)
            elif bsize != oldblocks[bva]:
                vw.delCodeBlock(bva)
                vw.addCodeBlock(bva, bsize, funcva)
            bcnt += 1
        except Exception as e:
            logger.warning('Codeblock analysis for 0x%.8x hit exception: %s', funcva, e)
            break

    vw.setFunctionMeta(funcva, 'Size', size)
    vw.setFunctionMeta(funcva, 'BlockCount', bcnt)
    vw.setFunctionMeta(funcva, 'InstructionCount', opcount)
    vw.setFunctionMeta(funcva, 'MnemDist', dict(mnem))



"""
Posix Signaling Module
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
import os
import sys
import struct
import signal
import logging


import vtrace
import vtrace.platforms.base as v_base

import Elf
from ctypes import *
import ctypes.util as cutil

import envi.cli as e_cli
import envi.symstore.resolver as e_resolv

logger = logging.getLogger(__name__)
libc = None

class PosixMixin:

    """
    A mixin for systems which use POSIX signals and
    things like wait()
    """

    def __init__(self):
        """
        Setup for the fact that we support signal driven
        debugging on posix platforms
        """
        self.stepping = False # Set this on stepi to diff the TRAP
        self.execing  = False # Set this on exec to diff the TRAP
        self.pthreads = [] # Some platforms make a pthread list

        self.fireTracerThread()

    def platformKill(self):
        self.sendSignal(signal.SIGKILL)

    def sendSignal(self, signo):
        self.requireAttached()
        os.kill(self.pid, signo)

    def platformSendBreak(self):
        self.sendSignal(signal.SIGTRAP)  # FIXME maybe change to SIGSTOP

    def platformWait(self):
        return os.waitpid(self.pid, 0)

    def handleAttach(self):
        self.fireNotifiers(vtrace.NOTIFY_ATTACH)
        self._findLibraryMaps(b'\x7fELF', always=True)
        self._simpleCreateThreads()
        # We'll emulate windows here and send an additional
        # break after our library load events to make things easy
        self.runAgain(False)  # Clear this, if they want BREAK to run, it will
        self.fireNotifiers(vtrace.NOTIFY_BREAK)
        # POSIX hack - Windows signals on library load, POSIX doesn't
        self._LibraryLoadHook()
        
    def _LibraryLoadHook(self):
        '''
        Implement at the platform level
        '''
        pass

    def platformProcessEvent(self, event):
        pid, status = event
        if os.WIFEXITED(status):
            tid = self.getMeta("ThreadId", -1)
            exitcode = os.WEXITSTATUS(status)
            if tid != self.getPid():
                # Set the selected thread ID to the pid cause
                # the old one's invalid
                if tid in self.pthreads:
                    self.pthreads.remove(tid)
                self.setMeta("ThreadId", self.getPid())
                self._fireExitThread(tid, exitcode)

            else:
                self._fireExit(exitcode)

        elif os.WIFSIGNALED(status):
            self._fireExit(os.WTERMSIG(status))

        elif os.WIFSTOPPED(status):
            sig = os.WSTOPSIG(status)
            self.handlePosixSignal(sig)

        else:
            logger.error("Unhandled posix status code: %d", status)

    def handlePosixSignal(self, sig):
        """
        Handle a basic posix signal for this trace.  This was seperated from
        platformProcessEvent so extenders could skim events and still use this logic.
        """
        if sig == signal.SIGTRAP:

            # Traps on posix systems are a little complicated
            if self.stepping:
                # FIXME try out was single step thing for intel
                self.stepping = False
                self._fireStep()

            elif self.checkWatchpoints():
                return

            elif self.checkBreakpoints():
                # It was either a known BP or a sendBreak()
                return

            elif self.execing:
                self.execing = False
                self.handleAttach()

            else:
                self._fireSignal(sig)

        elif sig == signal.SIGSTOP:
            # FIXME only on attaching..
            self.handleAttach()

        else:
            self._fireSignal(sig)


class ElfMixin:
    """
    A platform mixin to parse Elf binaries
    """
    def __init__(self):
        self.setMeta('Format', 'elf')

    def platformParseBinary(self, filename, baseaddr, normname):
        typemap = {
            Elf.STT_FUNC: e_resolv.FunctionSymbol,
            Elf.STT_SECTION: e_resolv.SectionSymbol,
        }

        try:
            fd = self.platformOpenFile(filename)
            elf = Elf.Elf(fd)
        except IOError:
            try:
                # it's possible we hit vdso or something similar
                elf = Elf.elfFromMemoryObject(self, baseaddr)
            except:
                raise
        # elf = Elf.Elf(fd)
        addbase = 0
        if not elf.isPreLinked() and elf.isSharedObject():
            addbase = baseaddr

        for sec in elf.sections:
            sym = e_resolv.SectionSymbol(sec.name, sec.sh_addr+addbase, sec.sh_size, normname)
            self.addSymbol(sym)

        for sym in elf.symbols:
            symclass = typemap.get((sym.st_info & 0xf), e_resolv.Symbol)
            sym = symclass(sym.name, sym.st_value+addbase, sym.st_size, normname)
            self.addSymbol(sym)

        for sym in elf.dynamic_symbols:
            symclass = typemap.get((sym.st_info & 0xf), e_resolv.Symbol)
            sym = symclass(sym.name, sym.st_value+addbase, sym.st_size, normname)
            self.addSymbol(sym)

        sym = e_resolv.Symbol('__entry', elf.e_entry+addbase, 0, normname)
        self.addSymbol(sym)

# As much as I would *love* if all the ptrace defines were the same all the time,
# there seem to be small platform differences...
# These are the ones upon which most agree
PT_TRACE_ME     = 0   # child declares it's being traced */
PT_READ_I       = 1   # read word in child's I space */
PT_READ_D       = 2   # read word in child's D space */
PT_READ_U       = 3   # read word in child's user structure */
PT_WRITE_I      = 4   # write word in child's I space */
PT_WRITE_D      = 5   # write word in child's D space */
PT_WRITE_U      = 6   # write word in child's user structure */
PT_CONTINUE     = 7   # continue the child */
PT_KILL         = 8   # kill the child process */
PT_STEP         = 9   # single step the child */

def ptrace(code, pid, addr, data):
    """
    The contents of this call are basically cleanly
    passed to the libc implementation of ptrace.
    """
    global libc
    if not libc:
        if os.getenv('ANDROID_ROOT'):
            cloc = '/system/lib/libc.so'

        else:
            cloc = cutil.find_library("c")
            if not cloc:
                raise Exception("ERROR: can't find C library on posix system!")

        libc = CDLL(cloc)
        libc.ptrace.restype = c_size_t
        libc.ptrace.argtypes = [c_int, c_uint32, c_size_t, c_size_t]
    return libc.ptrace(code, pid, c_size_t(addr), c_size_t(data))

class PtraceMixin:
    """
    A platform mixin for using the ptrace functions
    to attach/detach/continue/stepi etc. Many *nix systems
    will probably use this...

    NOTE: if you get a PT_FOO undefined, it *probably* means that
    the PT_FOO macro isn't defined for that platform (which means
    it need to be done another way like PT_GETREGS on darwin doesn't
    exist... but the darwin mixin over-rides platformGetRegs)
    """

    def __init__(self):
        """
        Setup supported modes
        """

        self.conthack = 0
        if sys.platform == "darwin":
            self.conthack = 1

        self.fireTracerThread()

    @v_base.threadwrap
    def platformExec(self, cmdline):
        self.execing = True
        cmdlist = e_cli.splitargs(cmdline)
        os.stat(cmdlist[0])
        pid = os.fork()
        if pid == 0:
            ptrace(PT_TRACE_ME, 0, 0, 0)
            os.execv(cmdlist[0], cmdlist)
            sys.exit(-1)
        return pid

    @v_base.threadwrap
    def platformWriteMemory(self, address, bytes):
        wordsize = len(struct.pack("P",0))
        remainder = len(bytes) % wordsize

        if remainder:
            pad = self.readMemory(address+(len(bytes)-remainder), wordsize)
            bytes += pad[remainder:]

        for i in range(int(len(bytes)/wordsize)):
            offset = wordsize * i
            dword = struct.unpack("L",bytes[offset:offset+wordsize])[0]
            if ptrace(PT_WRITE_D, self.pid, int(address+offset), int(dword)) != 0:
                raise Exception("ERROR ptrace PT_WRITE_D failed!")




qt_matrix = '''

QWidget {
    background-color: #000000;
    color: #00ff00;
    font: Courier;
}

QTextEdit {
    border: 2px solid #00802b;
}

QDockWidget::title {
    background: #000000;
    text-align: center;
}

QDockWidget::close-button, QDockWidget::float-button {
    border: 1px solid transparent;
    background: #00802b;
    padding: 0px;
}

QDockWidget::close-button:hover, QDockWidget::float-button:hover {
    background: #00ff00;
}

QTabBar::tab {
    /*
     background: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                 stop: 0 #E1E1E1, stop: 0.4 #DDDDDD,
                                 stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);
    */
    background: #000000;
    border: 2px solid #00802b;
    /* border-bottom-color: #C2C7CB; */
    border-top-left-radius: 4px;
    border-top-right-radius: 4px;
    min-width: 8ex;
    padding: 2px;
}

QTabBar::tab:selected {
    border: 3px solid #00ff00;
    /* border-bottom-color: #00ff00; */
}

QTreeView::item:selected {
     /* border: 1px solid #567dbc; */
    color: #000000;
    background: #00802b;
}

QMainWindow::seperator {
    width: 4px;
    height: 4px;
    background: #00802b;
}

QMainWindow::seperator:hover {
    width: 4px;
    height: 4px;
    background: #00ff00;
}

QMenu {
    color: #00ff00;
    background-color: #000000;

    border: 2px solid #00802b;

    selection-color: #000000;
    selection-background-color: #00ff00;
}

QLineEdit {
    border: 2px solid #00802b;
    border-radius: 4px;
    /* padding: 0 8px; */
    /* font: Courier; */
    /* color: #00ff00; */
    /* background: #000000; */
    selection-background-color: darkgray;
}

QTreeView {
    border: 2px solid #00802b;
    alternate-background-color: #0f0f0f;
}

QHeaderView::section {
     color: #00ff00;
     padding-left: 4px;
     background-color: #000000;
     border: 1px solid #00802b;
}

QPushButton {
     border-width: 2px;
     border-radius: 4px;
     border-style: outset;
     border-color: #00802b;
     padding: 1px;
}

QComboBox {
    padding: 1px;
    color: #00ff00;
    background: #000000;
    border-radius: 4px;
    border: 2px solid #00802b;
    selection-color: #000000;
    selection-background-color: #00ff00;
}

QMenuBar {
    color: #00ff00;
    background-color: #000000;
    selection-color: #000000;
    selection-background-color: #00ff00;
    border: 2px solid #00802b;
}

QMenuBar::item {
    color: #00ff00;
    background-color: #000000;
}

QMenuBar::item:selected {
    color: #00ff00;
    background-color: #000000;
}

QMainWindow::separator {
     width: 4px;
     height: 4px;
     background: #00802b;
}

QMainWindow::separator:hover {
     background: #00ff00;
}

QSplitter::handle {
     width: 6px;
     height: 6px;
     background: #000000;
}

QSplitter::handle:hover {
     background: #00ff00;
}

QToolButton:disabled {
    color: #606060;
}

QToolButton { /* all types of tool button */
    border: 2px solid #8f8f91;
    border-radius: 6px;
}

QToolBar {
    background: #101010;
    border: 2px solid #00802b;
    spacing: 3px; /* spacing between items in toolbar */
}
'''

defcolors = qt_matrix

def getDefaultColors():
    return qt_matrix




from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import *


class NodeColumn(QGraphicsItem):

    def __init__(self, vg, nodes, scene, left=None, right=None):
        QGraphicsItem.__init__(self)
        if scene is not None:
            scene.addItem(self)
        self.setZValue(-100)  # Get behind click events...

        self._v_vg = vg
        self._v_lines = []
        self._v_nodes = nodes

        # Remember if we srarted with something on our right or left
        self._v_goleft = (left is None)
        self._v_goright = (right is None)

        self._v_left = left
        self._v_right = right

        offset = 0
        for nid, nprops in nodes:
            txt = QGraphNode(vg, self, nid, nprops, scene=scene)
            txt.setY(offset)
            offset += txt.boundingRect().height()

    def takeOverView(self):
        self._removeLeft()
        self._removeRight()
        self.setX(0)
        self.setY(0)
        self._v_goleft = True
        self._v_goright = True

    def _removeLeft(self):
        left = self._v_left
        while left:
            nextleft = left._v_left
            left.removeColumn()
            left = nextleft

    def _removeRight(self):
        right = self._v_right
        while right:
            nextright = right._v_right
            right.removeColumn()
            right = nextright

    def removeColumn(self):
        scene = self.scene()
        for item in self._v_lines:
            scene.removeItem(item)
        scene.removeItem(self)

    def boundingRect(self):
        return self.childrenBoundingRect()

    def paint(self, x, y, z): pass

    def getRightBoundary(self):
        rect = self.boundingRect()
        return self.x() + rect.width()

    def getLeftBoundary(self):
        return self.x()

    def getYMid(self):
        return self.y() + (int(self.boundingRect().height()) >> 1)

    def drawLinesTo(self, colnode):
        '''
        Draw lines to our nodes from the specified one
        (used when we are on the right...)
        '''
        scene = self.scene()
        colpos = colnode.scenePos()
        colrect = colnode.boundingRect()

        ecolor = self._v_vg.getMeta('edgecolor', '#000')
        pen = QtGui.QPen(QtGui.QColor( ecolor))

        for item in self.childItems():
            itpos = item.scenePos()
            itrect = item.boundingRect()

            x1 = colpos.x() + colrect.width()
            y1 = colpos.y() + (int(colrect.height()) >> 1)

            x2 = itpos.x()
            y2 = itpos.y() + (int(itrect.height()) >> 1)

            lineitem = scene.addLine(x1, y1, x2, y2, pen=pen)
            self._v_lines.append(lineitem)

    def drawLinesFrom(self, colnode):
        '''
        Draw lines from our nodes to the specified one
        (used when we are on the left...)
        '''
        scene = self.scene()
        colpos = colnode.scenePos()
        colrect = colnode.boundingRect()

        ecolor = self._v_vg.getMeta('edgecolor', '#000')
        pen = QtGui.QPen(QtGui.QColor(ecolor))

        for item in self.childItems():
            itpos = item.scenePos()
            itrect = item.boundingRect()

            x1 = itpos.x() + itrect.width()
            y1 = itpos.y() + (int(itrect.height()) >> 1)

            x2 = colpos.x()
            y2 = colpos.y() + (int(colrect.height()) >> 1)

            lineitem = scene.addLine(x1, y1, x2, y2, pen=pen)
            self._v_lines.append(lineitem)

    def expandNode(self, colnode):

        mymid = colnode.scenePos().y()

        if self._v_goleft:

            self._removeLeft()

            nodes = [ (n1, self._v_vg.getNodeProps(n1)) for (eid, n1, n2, einfo) in self._v_vg.getRefsToByNid( colnode._v_nid ) ]

            col = NodeColumn( self._v_vg, nodes, self.scene(), right=self )
            hiswidth = col.boundingRect().width()
            col.setX( self.getLeftBoundary() - hiswidth - 20 )

            # Check if we want to scoot it down...
            hismid = col.getYMid()
            if hismid < mymid:
                col.setY( col.y() + (mymid - hismid) )

            col.drawLinesFrom( colnode )

            self._v_left = col

        if self._v_goright:

            self._removeRight()

            nodes = [ (n2, self._v_vg.getNodeProps(n2)) for (eid, n1, n2, einfo) in self._v_vg.getRefsFromByNid( colnode._v_nid ) ]

            col = NodeColumn( self._v_vg, nodes, self.scene(), left=self )
            col.setX( self.getRightBoundary() + 20 )

            # Check if we want to scoot it down...
            hismid = col.getYMid()
            if hismid < mymid:
                col.setY( col.y() + (mymid - hismid) )

            col.drawLinesTo( colnode )
            self._v_right = col

class QGraphNode(QGraphicsSimpleTextItem):

    def __init__(self, vg, column, nid, nprops, scene=None):
        QGraphicsSimpleTextItem.__init__(self, nprops.get('repr', 'node:{}'.format(nid) ), parent=column)
        if scene is not None:
            scene.addItem(self)

        self._v_vg = vg
        self._v_nid = nid
        self._v_nprops = nprops

        color = vg.getMeta('nodecolor', 'green')
        color = nprops.get('color', color)

        brush = QtGui.QBrush( QtGui.QColor( color ) )
        self.setBrush( brush )

    # TODO mouseDoubleClickEvent to take over center of view.

    def mousePressEvent(self, event):
        self.parentItem().expandNode( self )
        # Our scene's parent is a QGraphTreeView...
        self.scene().parent()._sig_NodeSelected.emit(self._v_nid, self._v_nprops)

    def contextMenuEvent(self, event):
        pos = event.screenPos()
        self.scene().parent()._sig_NodeContextMenu.emit(pos, self._v_nid, self._v_nprops)

class QGraphTreeView(QGraphicsView):

    _sig_NodeSelected = QtCore.pyqtSignal(object, dict)
    _sig_NodeContextMenu = QtCore.pyqtSignal( object, object, dict ) # pos, nid, nprops

    def __init__(self, vg, nodes, parent=None):
        QGraphicsView.__init__(self, parent=parent)
        scene = QGraphicsScene(parent=self)

        self.setScene( scene )
        self._v_nodecol = NodeColumn(vg, nodes, scene)
        self._v_vg = vg

    def loadNewGraph(self, vg, nodes):

        # setup default meta colors
        color = vg.getMeta('bgcolor', '#000000')
        self.scene().setBackgroundBrush( QtGui.QBrush( QtGui.QColor( color ) ) )

        self._v_nodecol._removeLeft()
        self._v_nodecol._removeRight()
        self._v_nodecol.removeColumn()

        self._v_nodecol = NodeColumn(vg, nodes, self.scene() )
        self._v_vg = vg

if __name__ == '__main__':

    import sys
    import visgraph.graphcore as vg_graphcore

    app = QApplication(sys.argv)
    app.setFont( QtGui.QFont('Courier') )

    initnodes = [ (i, {'repr':'node%d' % i}) for i in range( 30 ) ]

    # Build up a fake graph
    vg = vg_graphcore.Graph()
    for nid,nprops in initnodes:
        vg.addNode(nodeid=nid, repr='node %d' % nid)

    for nid,nprops in initnodes:
        for knid,knprops in initnodes:
            if nid == knid:
                continue
            vg.addEdge(nid, knid)

    win = QGraphTreeView( vg, initnodes, parent=None )

    win.show()
    app.exec_()



import os
import sys
import time
import unittest
import subprocess

import vtrace

import envi.tests as e_test


class VtraceProcessTest(unittest.TestCase):

    pypath = os.path.join('vtrace', 'tests', 'mains', 'main.py')

    @e_test.skip('darwin')
    def setUp(self):
        self.exitrun = False
        self.proc = subprocess.Popen([sys.executable, self.pypath], stdin=subprocess.PIPE, stdout=subprocess.PIPE, universal_newlines=True)
        assert(self.proc.stdout.readline().strip() == 'testwait')
        self.trace = vtrace.getTrace()
        self.trace.attach(self.proc.pid)

    def tearDown(self):
        retry = 0
        while retry < 5:
            if not self.exitrun and self.trace.isAttached():
                self.trace.setMode('RunForever', True)
                self.proc.stdin.write('testmod\n')
                self.proc.stdin.flush()
                self.trace.run()
            try:
                self.proc.wait(timeout=10)
            except subprocess.TimeoutExpired:
                pass
            finally:
                retry += 1

        if self.proc.returncode is None:
            # FINE. shoot the process and keep going. Ain't nobody got time for that.
            self.proc.kill()
        try:
            self.proc.stdout.close()
            self.proc.stdin.close()
        except:
            pass
        self.trace.release()

    def runProcess(self):
        self.trace.setMode('RunForever', True)
        self.trace.setMode('NonBlocking', True)
        self.proc.stdin.write('testmod\n')
        self.proc.stdin.flush()
        self.trace.run()

    def runUntilExit(self):
        self.exitrun = True
        self.trace.setMode('RunForever', True)
        self.trace.setMode('NonBlocking', False)
        self.proc.stdin.write('testmod\n')
        self.proc.stdin.flush()
        self.trace.run()
        time.sleep(1)

        self.assertEqual(self.trace.getMeta('ExitCode'), 33)


class VtraceExecTest(VtraceProcessTest):

    pypath = os.path.join('vtrace', 'tests', 'mains', 'mainexec.py')

    @e_test.skip('darwin')
    def setUp(self):
        self.exitrun = False
        self.trace = vtrace.getTrace()
        self.trace.execute('%s %s' % (sys.executable, self.pypath))
        self.trace.requireAttached()

    def tearDown(self):
        if self.trace.isAttached():
            self.trace.setMode('RunForever', True)
            self.trace.setMode('NonBlocking', False)
            self.trace.run()
        self.trace.release()

    def runUntilExit(self):
        self.exitrun = True
        self.trace.setMode('RunForever', True)
        self.trace.setMode('NonBlocking', False)
        self.trace.run()

        self.assertEqual(self.trace.getMeta('ExitCode'), 33)

    def runProcess(self):
        self.trace.setMode('RunForever', True)
        self.trace.setMode('NonBlocking', True)
        self.trace.run()



'''
More comprehensive tests. Not necessarily for correctness, but mostly so that
if things do change, we're aware of it.

Binaries used here should be smaller so analysis completes in a reasonable enough
timeframe (since we'll be doing full analysis)
'''
import os
import json
import logging
import unittest

import vivisect
import vivisect.tests.helpers as helpers

from vivisect.const import *


logger = logging.getLogger(__name__)

class StabilityTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        self.testfiles = (
            ('redxor.stability.json', ('linux', 'amd64', 'redxor.malware')),
            ('sudo.stability.json', ('linux', 'amd64', 'sudo')),
        )

    def _cmp_func(self, file, base, vw):
        fails = []
        # TODO: There's some instability in the 3rd element for certain codeblocks when
        # a function is jmp'd to by other functions. It's not horrifically bad, but
        # determinism is nice, so hunt down a way to fix that
        for fva, (size, instcount, blocks) in base['functions'].items():
            fva = int(fva)
            newfva = vw.getFunction(fva)
            if not newfva:
                fails.append((fva, 'fva', 'New version not defined'))
            try:
                self.assertEqual(fva, newfva)
                self.assertEqual(size, vw.getFunctionMeta(fva, 'Size'))
                self.assertEqual(instcount, vw.getFunctionMeta(fva, 'InstructionCount'))
            except Exception as e:
                fails.append((fva, 'meta', str(e)))
                continue

        basefunc = set([int(x) for x in base['functions'].keys()])
        for fva in vw.getFunctions():
            if fva not in basefunc:
                fails.append((fva, 'fva', 'Old version not defined'))
                continue
            size, instcount, blocks = base['functions'][str(fva)]
            try:
                self.assertEqual(size, vw.getFunctionMeta(fva, 'Size'))
                self.assertEqual(instcount, vw.getFunctionMeta(fva, 'InstructionCount'))
            except Exception as e:
                fails.append((fva, 'meta', str(e)))
                continue

        if fails:
            for fva, sect, mesg in fails:
                logger.error('[ %s ] Test fail for 0x%.8x in section %s: %s' % (file, fva, sect, mesg))
            self.fail("Stability Function Tests Failed. See logs for details.")

    def _cmp_loc(self, file, name, base, new):
        baseonly = []
        newonly = []

        fixed = []
        for b in base:
            if type(b) is list:
                b = tuple(b)
            fixed.append(b)

        for loc in fixed:
            if loc not in new:
                baseonly.append(loc)

        for loc in new:
            if loc not in fixed:
                newonly.append(loc)

        if baseonly:
            self.fail("[%s : %s] Analysis removed some locations. Here's the ones that got removed: %s" % (file, name, baseonly))

        if newonly:
            self.fail("[%s : %s] Analysis added some locations. Here's the new ones: %s" % (file, name, newonly))

    def _get_strtype(self, base):
        retn = []
        for sloc in base:
            va, size, ltyp, subs = sloc
            nsubs = set()
            for sub in subs:
                nsubs.add(tuple(sub))
            retn.append((va, size, ltyp, nsubs))

        return retn

    def _compare(self, filename, base, vw):
        testhooks = {
            'entrypoints': (self._cmp_loc, vw.getEntryPoints()),

            'pointers': (self._cmp_loc, vw.getLocations(LOC_POINTER)),
            'strings': (self._cmp_loc, self._get_strtype(vw.getLocations(LOC_STRING))),
            'unicode': (self._cmp_loc, self._get_strtype(vw.getLocations(LOC_UNI))),
            'imports': (self._cmp_loc, vw.getImports()),
            'exports': (self._cmp_loc, vw.getExports()),
            'names': (self._cmp_loc, vw.getNames()),
            'xrefs': (self._cmp_loc, vw.getXrefs()),
            'relocations': (self._cmp_loc, vw.getRelocations()),
        }

        self._cmp_func(filename, base, vw)

        for name, (func, new) in testhooks.items():
            if name == 'strings' or name == 'unicode':
                basedata = self._get_strtype(base[name])
            else:
                basedata = base[name]
            func(filename, name, basedata, new)

    def test_features(self):
        dirn = os.path.dirname(__file__)
        for filename, binpath in self.testfiles:
            testpath = os.path.join(dirn, 'stabilitydata', filename)
            with open(testpath, mode='r', encoding='utf-8') as fd:
                base = json.load(fd)
            vw = helpers.getTestWorkspace(*binpath)
            self._compare(filename, base, vw)



'''
Paths are enumerated threads through a particular graph.  They
are implemented as an optimized hierarchical graph using python
primitives to save memory and processing time...

Each "leaf" node may be tracked back for it's entire path by
tracking back to parents.
'''

def newPathNode(parent=None, **kwargs):
    '''
    Create a new path node with the given properties
    '''
    ret = (parent, [], kwargs)
    if parent is not None:
        parent[1].append(ret)
    return ret

def getNodeParent(pnode):
    return pnode[0]

def delPathNode(pnode):
    '''
    Prune (remove) this node from the parent...
    '''
    p = getNodeParent(pnode)
    if p is not None:
        p[1].remove(pnode)

def getNodeIndex(pnode):
    p = getNodeParent(pnode)
    if p is not None:
        return p[1].index(pnode)
    return None

def getNodeKids(pnode):
    '''
    Return the list of children nodes for the specified
    node.

    Example: for knode in getNodeKids(pnode):
    '''
    return pnode[1]

def getRootNode(pnode):
    '''
    Get the root node for the path tree which contains
    pnode.

    Example: root = getRootNode(branchnode)
    '''
    ret = pnode
    while pnode[0] is not None:
        pnode = pnode[0]
    return pnode

def getLeafNodes(pnode):
    '''
    Get all the leaf nodes for the path tree which contains
    the pnode.

    Example: for leaf in getLeafNodes(root):
    '''
    root = getRootNode(pnode)
    ret = []
    todo = [root, ]
    while len(todo):
        x = todo.pop()
        if len(x[1]) == 0:
            ret.append(x)
            continue
        for n in x[1]:
            todo.append(n)
    return ret

def getPathToNode(pnode):
    '''
    Return a list of the path nodes which lead from the
    root node to the specified path node.
    '''
    path = []
    while pnode is not None:
        path.append(pnode)
        pnode = pnode[0]

    path.reverse()
    return path

def getAllPaths(pnode):
    '''
    Get a list of lists which has each path flattened out.

    Example: for path in getAllPaths(pnode):
                 for node in path:
                    doStuff()
    '''
    leafs = getLeafNodes(pnode)
    paths = []
    for leaf in leafs:
        path = getPathToNode(leaf)
        paths.append(path)
    return paths

def getNodeProp(pnode, key, default=None):
    '''
    Get a property from the given node, returning
    default in the case that the specified property is
    not present.

    Example:
        name = getNodeProp(pnode, 'name', 'Unknown')
    '''
    return pnode[2].get(key, default)

def getPathProp(pnode, key, default=None):
    '''
    Retrieve the specified property by walking the give
    path backward until the property is found.  Returns
    the specified default if the specified key is not found
    as a property of any node in the given path.

    Example:
        name = getPathProp(pnode, 'name', 'Unknown')
    '''
    parent = pnode
    while parent is not None:
        parent, kids, props = parent
        x = props.get(key)
        if x is not None:
            return x
    return default

def setNodeProp(pnode, key, value):
    '''
    Set a spcified property on the given path node.

    Example:
        setNodeProp(pnode, 'name', 'woot')
    '''
    pnode[2][key] = value

def isPathLoop(pnode, key, value):
    '''
    Assuming you have some identifier property (such as graph node id)
    being set on the path nodes, you may use this API to determine if
    the current path has a node with the specified key/value property.

    Example:
        if searchPathLoop(pnode, 'nid', 5):
            continue
    '''
    parent = pnode
    while parent is not None:
        parent, kids, props = parent
        if props.get(key) == value:
            return True
    return False

def getPathLoopCount(pnode, key, value):
    '''
    Assuming that the key is unique, walk the current path and see how
    many times "key" has the specified value.  This will be how many instances
    of a loop have been encountered.
    '''
    count = 0
    parent = pnode
    while parent is not None:
        parent, kids, props = parent
        if props.get(key) == value:
            count += 1
    return count


def trimPath(pnode):
    '''
    Doing analysis on a path tree can be very memory consuming.  If an analysis tool is
    powering through bajillions of nodes, the tree can consume all RAM.  However,
    the analysis may be doable with some selective trimming of the path tree.
    Only trim when you are done with a path node, and make sure your algorithm uses
    newPathNode on all "children" before traversing into one of them.

    trimPath() will remove pnode from it's parent node, then check if the parent node
    has any other children nodes, and recursively remove path links to all unnecessary
    path nodes leading up to pnode.

    caveat: if you are not calling newPathNode (which adds a reference from it's parent)
    on all child nodes before traversing into any of them, it is possible that the
    current node could be trimmed because it looks like it doesn't have any children.
    '''
    while True:
        p = getNodeParent(pnode)

        # if we don't have a parent, we're done
        if p is None:
            break

        # remove our burden from our parent
        if pnode in p[1]:
            p[1].remove(pnode)

        # if our parent still has kids living at home, we're done
        if len(p[1]):
            break

        # time to kill our parent
        pnode = p


def reprPath(node, startFromRoot=True):
    r = ''

    if startFromRoot:
        node = getRootNode(node)

    snid = getNodeProp(node, 'nid')
    todo = [(snid, node, 1)]
    r += hex(snid)
    r += '\n'

    while len(todo):
        nid, pnode, indent = todo.pop()

        for tpnode in pnode[1]:
            tnid = getNodeProp(tpnode, 'nid')
            outstr = "   |"*indent + "--" + hex(tnid)
            if tnid == snid:
                outstr += "  <<<<-- our node!"
            r += outstr
            r += '\n'

            todo.append((tnid, tpnode, indent + 1))

    return r



import os
import sys
import time
import queue
import logging
import argparse
import threading

import cobra
import cobra.dcode

import envi.config as e_config
import envi.common as e_common
import envi.threads as e_threads

import vivisect
import vivisect.cli as v_cli
import vivisect.parsers as v_parsers
import vivisect.storage.basicfile as viv_basicfile

from vivisect.const import *
from vivisect.defconfig import *

logger = logging.getLogger(__name__)

viv_port = 0x4074

viv_s_ip = '224.56.56.56'
viv_s_port = 26998

# DEFAULTS for non-configured use
timeout_wait = 10
timeout_sock = 50
timeout_aban = 120   # 2 minute timeout for abandon

# This should *only* rev when they're truly incompatible
server_version = 20130820


class VivServerClient:
    '''
    Implement "glue" methods for the vivisect workspace client to
    talk to the server...
    '''
    def __init__(self, vw, server, wsname):
        self.vw = vw
        self.chan = None
        self.wsname = wsname
        self.server = server
        self._processlock = threading.Lock()
        self.eoffset = 0
        self.q = queue.Queue()  # The actual local Q we deliver to
        self._qtime = time.time()
        self._qcount = 0
        self._qcountfull = 0
        self._starttime = time.time()

    @e_threads.firethread
    def _eatServerEvents(self):
        while True:
            try:
                logger.debug("Starting a Workspace Event Processing run...")
                # Fetch events OUTSIDE the lock so waitForCurEvents() isn't
                # starved while the server-side ChunkQueue.get() blocks for
                # up to timeout_wait seconds.
                events = self.server.getNextEvents(self.chan)

                count = 0
                with self._processlock:
                    for event in events:
                        count += 1
                        self.q.put(event)

                logger.debug("Workspace Event Processing run Complete. (%d events processed)", count)
                time.sleep(.01)

            except Exception:
                logger.warning("Error in _eatServerEvents (will retry)", exc_info=1)
                time.sleep(.5)

    def vprint(self, msg):
        return self.server.vprint(msg)

    def _fireEvent(self, event, einfo, local=False, skip=None):
        return self.server._fireEvent(self.chan, event, einfo, local=local, skip=skip)

    def createEventChannel(self):
        self.chan = self.server.createEventChannel(self.wsname)
        self._monitorEventQueuePerf()
        logger.debug("VivServerClient: Event Channel created, firing _eatServerEvents() in separate thread")
        self._eatServerEvents()
        logger.debug("Waiting for initial Workspace Events to finish processing")
        self.waitForCurEvents()
        logger.debug("Done processing initial Workspace Events, now back to our program")

        return self.chan

    def waitForCurEvents(self):
        '''
        Since _eatServerEvents() runs in its own thread, we just wait for it's
        _processlock to be released, signaling that it's done processing that
        batch of VivEvents.  We don't need to do anything with the lock, so 
        we just release the lock and return
        '''
        logger.debug("waiting for current Workspace Event run to complete...")
        with self._processlock:
            pass

    def exportWorkspace(self):
        # Retrieve the big initial list of viv events
        return self.server.getNextEvents(self.chan)

    @e_threads.firethread
    def _monitorEventQueuePerf(self):
        self._starttime = time.time()
        while True:
            qlen = -1
            try:
                time.sleep(1)
                q = self.q

                if q:
                    qlen = q.qsize()
                    if qlen % 100 == 1:
                        logger.debug("qlen: %d (%d/%d)", qlen, self._qcount, self._qcountfull)

                curtime = time.time()
                delta = curtime - self._qtime
                if delta > 1 and self._qcount:
                    self._qtime = curtime

                    self._qrate = self._qcount / delta
                    logger.info("Event Queue Rate: %d/sec (%d / %d)", self._qrate, delta, (self._qtime-self._starttime))
                    logger.debug("      qlen: %d (%d/%d)", qlen, self._qcount, self._qcountfull)

                    # reset counter
                    self._qcount = 0
            except:
                logger.warning("Problem in Monitor Thread", exc_info=1)


    def waitForEvent(self, chan, timeout=None):
        q = self.q
        qitem = q.get(timeout=timeout)
        self._qcount += 1
        self._qcountfull += 1
        return qitem

    def getLeaderLocations(self):
        try:
            return self.server.getLeaderLocations(self.wsname)
        except Exception as e:
            logger.warning("error in getLeaderLocations(): %r (is server up to date?)" % e)
            return {}

    def getLeaderSessions(self):
        try:
            return self.server.getLeaderSessions(self.wsname)
        except Exception as e:
            logger.warning("error in getLeaderSessions(): %r (is server up to date?)" % e)
            return {}


class VivServer:

    def __init__(self, dirname='', **kwargs):
        self.path = os.path.abspath(dirname)

        cfgdir = kwargs.get('confdir', None)
        if cfgdir:
            self.vivhome = os.path.abspath(cfgdir)
        else:
            self.vivhome = e_config.gethomedir(".viv", makedir=True)
        cfgpath = os.path.join(self.vivhome, 'viv.json')
        self.config = e_config.EnviConfig(filename=cfgpath, defaults=defconfig, docs=docconfig, autosave=True)

        self.timeout_wait = self.config.viv.remote.timeout_wait
        self.timeout_aban = self.config.viv.remote.timeout_aban
        self.wsdict = {}
        self.chandict = {}
        self.wslock = threading.Lock()

        self._loadWorkspaces()
        self._maintThread()
        self._saveWorkspaceThread()

    def vprint(self, msg):
        print(msg)

    def getServerVersion(self):
        return server_version

    @e_threads.maintthread(1)
    def _maintThread(self):

        for chan in list(self.chandict.keys()):
            chaninfo = self.chandict.get(chan)
            # NOTE: double check because we're lock free...
            if chaninfo is None:
                continue

            wsinfo, queue, chanleaders = chaninfo
            if queue.abandoned(self.timeout_aban):
                self.cleanupChannel(chan)

    @e_threads.maintthread(30)
    def _saveWorkspaceThread(self):
        for wsinfo in self.wsdict.values():
            lock, path, events, users, leaders, leaderloc = wsinfo
            if events:
                with lock:
                    wsinfo[2] = []  # start a new events list...

                viv_basicfile.vivEventsAppendFile(path, events)

    def _req_wsinfo(self, wsname):
        wsinfo = self.wsdict.get(wsname)
        if wsinfo is None:
            raise Exception('Invalid Workspace Name: %s' % wsname)
        return wsinfo

    def listWorkspaces(self):
        '''
        Get a list of the workspaces this server is willing to share.
        '''
        return list(self.wsdict.keys())

    def addNewWorkspace(self, wsname, events):
        wspath = os.path.join(self.path, wsname)
        wspath = os.path.realpath(wspath)
        if not wspath.startswith(self.path):
            raise Exception('Nice try liberty... ;)')

        if os.path.exists(wspath):
            raise Exception('Workspace %s already exists!' % wsname)

        wsdir = os.path.dirname(wspath)
        if not os.path.isdir(wsdir):
            os.makedirs(wsdir, 0o750)

        viv_basicfile.vivEventsToFile(wspath, events)
        wsinfo = [threading.Lock(), wspath, [], {}, {}, {}]
        self.wsdict[wsname] = wsinfo

    def _loadWorkspaces(self):

        # First, ditch any that are missing
        for wsname in self.wsdict.keys():
            wsinfo = self.wsdict.get(wsname)
            if not os.path.isfile(wsinfo[1]):
                self.wsdict.pop(wsname, None)

        for dirname, dirnames, filenames in os.walk(self.path):
            for filename in filenames:
                wspath = os.path.join(dirname, filename)
                wsname = os.path.relpath(wspath, self.path)

                if not os.path.isfile(wspath):
                    continue

                with open(wspath, 'rb') as f:
                    ext = f.read(6)

                if not ext:
                    continue

                if not v_parsers.guessFormat(ext) in ('viv', 'mpviv'):
                    continue

                wsinfo = self.wsdict.get(wsname)
                if wsinfo is None:
                    # Initialize the workspace info tuple
                    lock = threading.Lock()
                    wsinfo = [lock, wspath, [], {}, {}, {}]
                    logger.debug('loaded: %s', wsname)
                    self.wsdict[wsname] = wsinfo

    def getNextEvents(self, chan):
        chaninfo = self.chandict.get(chan)
        if chaninfo is None:
            raise Exception('Invalid Channel: %s' % chan)
        logger.debug("getNextEvents(%r)", repr(chaninfo))
        return chaninfo[1].get(timeout=self.timeout_wait)

    # All APIs from here down are basically mirrors of the workspace APIs
    # used with remote workspaces, with a prepended chan first argument

    def _fireEvent(self, chan, event, einfo, local=False, skip=None):
        #print("_fireEvent: %r %r %r %r %r" % (chan, event, einfo, local, skip))

        if chan in self.chandict:
            wsinfo, q, chanleaders = self.chandict.get(chan)
            lock, fpath, pevents, users, leaders, leaderloc = wsinfo
            oldclient = False

        elif chan in self.wsdict:
            # DEPRECATED: this is for backwards compat.  use only the chandict code one year from today, 5/10/2022.
            wsname = chan
            wsinfo = self._req_wsinfo(wsname)
            lock, fpath, pevents, users, _, _ = wsinfo
            # cheat for older clients... they don't get all the follow-the-leader goodness until they upgrade
            chanleaders = []
            leaders = {}
            leaderloc = {}
            oldclient = True

        else:
            raise Exception("BAD CHANNEL: _fireEvent: %r %r %r %r %r" % (chan, event, einfo, local, skip))

        evtup = (event, einfo)
        with lock:
            # Transient events do not get saved
            if not event & VTE_MASK:
                pevents.append(evtup)

            else:
                vtevent = event ^ VTE_MASK
                if vtevent == VTE_FOLLOWME:
                    pass

                elif vtevent == VTE_IAMLEADER:
                    logger.info("VTE_IAMLEADER: %r" % repr(evtup))
                    uuid, user, fname, locexpr = einfo
                    leaders[uuid] = (user, fname)
                    leaderloc[uuid] = locexpr
                    chanleaders.append(uuid)

                elif vtevent == VTE_FOLLOWME:
                    logger.info("VTE_FOLLOWME: %r" % repr(evtup))
                    uuid, expr = einfo
                    leaderloc[uuid] = expr

                elif vtevent == VTE_KILLLEADER:
                    logger.info("VTE_KILLLEADER: %r" % repr(evtup))
                    uuid = einfo
                    leaders.pop(uuid, None)
                    leaderloc.pop(uuid, None)

                elif vtevent == VTE_MODLEADER:
                    logger.info("VTE_MODLEADER: %r" % repr(evtup))
                    uuid, user, fname = einfo
                    leaders[uuid] = (user, fname)


            # SPEED HACK
            [q.append(evtup) for chan, q in users.items() if chan != skip]

    def createEventChannel(self, wsname):
        wsinfo = self._req_wsinfo(wsname)
        chan = e_common.hexify(os.urandom(16))
        chunksize = self.config.viv.server.queue_chunksize

        # Load up the queue with events from the file, and current events
        lock, fpath, pevents, users, leaders, leaderloc = wsinfo

        with lock:
            # These must reference the same actual list object...
            queue = VivChunkQueue(chunksize=chunksize)
            users[chan] = queue
            chanleaders = []
            self.chandict[chan] = [wsinfo, queue, chanleaders]

            # add events after the channel is created.  that way the client can start pulling event groups immediately
            fileevts = viv_basicfile.vivEventsFromFile(fpath)
            fileevts.extend(pevents)
            # use this firethread function to add events while we return the channel
            queue.chunkedAddLargeEventList(fileevts)

        return chan

    def getLeaderLocations(self, wsname):
        wsinfo = self._req_wsinfo(wsname)
        lock, path, events, users, leaders, leaderloc = wsinfo
        return dict(leaderloc)

    def getLeaderSessions(self, wsname):
        wsinfo = self._req_wsinfo(wsname)
        lock, path, events, users, leaders, leaderloc = wsinfo
        return dict(leaders)

    def cleanupChannel(self, chan):
        chantup = self.chandict.get(chan, None)
        if chantup is None:
            logger.warning("Attempting to clean up nonexistent channel: %r" % chan)
            return

        # Remove all channels originating from this channel
        wsinfo, queue, chanleaders = chantup
        for uuid in chanleaders:
            self._fireEvent(chan, VTE_KILLLEADER | VTE_MASK, uuid)

        # Remove from the workspace clients
        lock, fpath, pevents, users, leaders, leaderloc = wsinfo
        with lock:
            userinfo = users.pop(chan, None)

        # Remove from our chandict
        self.chandict.pop(chan, None)

# TODO: queue events to a list...  after a few ms (or CHUNKSIZE reached), convert to a Dict/List and send it?

class VivChunkQueue(e_threads.ChunkQueue):
    '''
    Vivisect Event specific version of the ChunkQueue
    This ChunkQueue implements ChunkQueue helpers specifically designed to
    handle VivServer requirements, namely:
    1) Chunking Around VWE_ADDMMAP events to reduce delays between request to
        response
    2) Front-loading chunking of the initial file VivServer download. Again,
        this helps avoid timeouts while processing large workspaces.  With
        the addition of multi-library VivWorkspaces, this can be a *real*
        problem.  VivChunkQueue helps solve those problems
    3) Introduction of the Event Group, enabling the functionality required
        to do #2 efficiently.  Front-loading pre-analyzed groups of events
        speeds up loading dramatically
    '''
    def _get_items(self):
        '''
        Already has self.lock when called.
        '''
        if not len(self.items):
            return []

        # watch for event groups
        # "event groups" are a dict with a list stored at key 0.  this list is
        # then able to be shoved over the Cobra channel immediately, front-loading
        # any preparation, and drastically speeding up initial message glut on
        # workspace loading.
        if type(self.items[0]) == dict:
            evtgroup = self.items.pop(0)
            evts = evtgroup[0]
            logger.debug("_get_items: STORED EVENT GROUP of size %d (queue remaining: %d)", len(evts), len(self.items))
            return evts

        # handle events in "chunks"
        if self.chunksize:
            logger.debug('_get_items(): chunksize: %d', self.chunksize)

            # search for ADDMMAPs so we don't group two in one transmission
            # RAM and CPU utilization make larger workspaces time out otherwise
            # especially if there are multiple large memory maps (eg. multi-lib
            # workspaces)
            idx = None
            for idx in range(self.chunksize-1):
                # break if we reach the end of the event list
                if idx >= len(self.items):
                    break

                evtitem = self.items[idx]

                # search for Event Groups
                if type(evtitem) == dict:
                    # don't include this in the list!  kick out and let the
                    # "event group" section (above) handle this one
                    idx -= 1
                    break

                evtype, evtdata = evtitem
                # search for ADDMMAP events
                if evtype == vivisect.VWE_ADDMMAP:
                    break


            # idx+1 to grabs the last item identified (except for Event Groups)
            ret = self.items[:idx+1]

            numEaten = len(ret)
            # just rebuild the list - tried several options, this still won out
            self.items = self.items[numEaten:]

            numLeft = len(self.items)
            logger.debug('_get_items() - returning (%d eaten / %d left)', numEaten, numLeft)

        else:
            # if chunksize is 0 or None, go old-school and just shove it all in
            ret = self.items
            for idx, (evtype, evtdata) in enumerate(ret):
                # still want to only send one memory map at a time
                if evtype == vivisect.VWE_ADDMMAP:
                    ret = self.items[:idx+1]
                    break

            self.items = self.items[len(ret):]
            logger.debug("_get_items() - returning %d items (%d remaining)", len(ret), len(self.items))
        return ret

    @e_threads.firethread
    def chunkedAddLargeEventList(self, evts):
        '''
        Break up a large list of events into appropriately pre-sized chunks,
        allowing for high efficiency in serving them.  This is a @firethread
        function, meaning that it runs in its own thread, making it "fire-and-
        forget".
        '''
        evtcount = len(evts)
        idx = 0
        logger.debug("... adding %d events to the Queue", evtcount)
        while evtcount > 0:
            anchor = idx
            for count in range(self.chunksize):
                # pick and poke...
                evt = evts[idx]
                idx += 1
                evtcount -= 1
                if evtcount <10:
                    logger.debug("...evtcount==%d, idx==%d", evtcount, idx)
                if evtcount == 0:
                    # we don't want to clip the last entry
                    idx += 1
                    break


                # if it's a memory map, only include one per chunk
                if evt[0] == vivisect.VWE_ADDMMAP:
                    logger.debug(" (chopping after VWE_ADDMMAP)")
                    break

            # here is the list of events we're packaging up
            worklist = evts[anchor:idx]

            # we package the list in a dict, to differentiate between normal events and event groups
            workdict = {0 : worklist}

            # add this group to the Queue
            logger.debug("... appending %d event chunk to the Queue (%d:%d)", len(worklist), anchor, idx)
            self.append(workdict)



def getServerWorkspace(server, wsname):
    vw = v_cli.VivCli()
    logger.debug("Starting:  VivServerClient(%r, %r)", server, wsname)
    cliproxy = VivServerClient(vw, server, wsname)
    logger.debug("Starting:  vw.initWorkspaceClient()")
    vw.initWorkspaceClient(cliproxy)
    logger.debug("Complete:  vw.initWorkspaceClient()")
    return vw


def connectToServer(hostname, port=viv_port):
    builder = cobra.initSocketBuilder(hostname, port)
    builder.setTimeout(timeout_sock)
    server = cobra.CobraProxy("cobra://%s:%d/VivServer" % (hostname, port), msgpack=True)
    version = server.getServerVersion()
    if version != server_version:
        raise Exception('Incompatible Server: his ver: %d our ver: %d' % (version, server_version))
    return server


def runMainServer(dirname='', port=viv_port):
    s = VivServer(dirname=dirname)
    daemon = cobra.CobraDaemon(port=port, msgpack=True)
    daemon.recvtimeout = timeout_sock
    daemon.shareObject(s, 'VivServer')
    daemon.serve_forever()


def setup():
    ap = argparse.ArgumentParser('Start a vivisect server to share workspaces')
    ap.add_argument('dirn', help='A directory full of *.viv files to share')
    ap.add_argument('--port', '-p', type=int, default=viv_port,
                    help='The port to start server on (defaults to %d)' % viv_port)
    return ap


def main():
    opts = setup().parse_args()
    vdir = os.path.abspath(opts.dirn)
    if not os.path.isdir(vdir):
        logger.error('%s is not a valid directory!', vdir)
        return -1

    print(f'VivServer starting (port: {opts.port})')
    runMainServer(vdir, opts.port)


if __name__ == '__main__':
    sys.exit(main())



import unittest

import visgraph.graphcore as v_graphcore
import visgraph.layouts.dynadag as v_dynadag

class GraphLayoutTest(unittest.TestCase):

    def sampGraph1(self):
        g = v_graphcore.HierGraph()

        g.addNode('a',rootnode=True)
        g.addNode('b')
        g.addNode('c')
        g.addNode('d')

        g.addEdgeByNids('a','b')
        g.addEdgeByNids('a','c')
        g.addEdgeByNids('c','a') # loop...
        g.addEdgeByNids('c','d')

        return g

    def sampGraph2(self):
        g = v_graphcore.HierGraph()

        g.addNode('a',rootnode=True)
        g.addNode('b')
        g.addNode('c')
        g.addNode('d')
        g.addNode('e')

        #       a
        #     / | \
        #   b  +c  |
        #      | \ /
        #      |  d
        #      |  |
        #      |  e
        #      +--+

        g.addEdgeByNids('a','b')
        g.addEdgeByNids('a','c')
        g.addEdgeByNids('a','d') # skip layer
        g.addEdgeByNids('c','d')
        g.addEdgeByNids('d','e')
        g.addEdgeByNids('e','c') # loop skip

        return g

    def test_visgraph_weights(self):

        g1 = self.sampGraph1()
        g1weights = {'a':0,'b':1,'c':1,'d':2}
        self.assertEqual(g1.getHierNodeWeights(), g1weights)

        for nid,nprops in g1.getNodes():
            self.assertEqual(nprops.get('weight'),g1weights.get(nid))

        g2 = self.sampGraph2()
        g2weights = {'a':0,'b':1,'c':3,'d':2,'e':3}
        self.assertEqual(g2.getHierNodeWeights(), g2weights)

        for nid,nprops in g2.getNodes():
            self.assertEqual(nprops.get('weight'),g2weights.get(nid))

    def test_visgraph_dynadag(self):
        g1 = self.sampGraph1()
        lyt = v_dynadag.DynadagLayout(g1)
        lyt.layoutGraph()

        self.assertEqual(g1.getNode('a')[1].get('position'),(20,0))
        self.assertEqual(g1.getNode('b')[1].get('position'),(20,40))
        self.assertEqual(g1.getNode('c')[1].get('position'),(40,40))
        self.assertEqual(g1.getNode('d')[1].get('position'),(20,80))

        g2 = self.sampGraph2()
        lyt = v_dynadag.DynadagLayout(g2)
        lyt.layoutGraph()

        self.assertEqual(g2.getNode('a')[1].get('position'),(20,0))
        self.assertEqual(g2.getNode('b')[1].get('position'),(20,40))
        self.assertEqual(g2.getNode('c')[1].get('position'),(40,120))
        self.assertEqual(g2.getNode('d')[1].get('position'),(20,80))
        self.assertEqual(g2.getNode('e')[1].get('position'),(20,120))




ord_names = {
    100:'ThunRTMain',
    101:'VBDllUnRegisterServer',
    102:'VBDllCanUnloadNow',
    103:'VBDllRegisterServer',
    104:'VBDllGetClassObject',
    105:'UserDllMain',
    106:'DllRegisterServer',
    107:'DllUnregisterServer',
    108:'__vbaAryLock',
    109:'__vbaBoolErrVar',
    110:'__vbaRedimVar2',
    111:'__vbaStrErrVarCopy',
    112:'__vbaVarLateMemCallLd',
    113:'__vbaVarLateMemCallLdRf',
    114:'__vbaVarLateMemCallSt',
    115:'__vbaVarLateMemSt',
    116:'__vbaVarLateMemStAd',
    117:'__vbaAryVarVarg',
    118:'__vbaFpCDblR4',
    119:'__vbaFpCDblR8',
    120:'__vbaFpCSngR4',
    121:'__vbaFpCSngR8',
    122:'__vbaFpCmpCy',
    123:'__vbaFpCy',
    124:'__vbaFpI2',
    125:'__vbaFpI4',
    126:'__vbaFpR4',
    127:'__vbaFpR8',
    128:'__vbaFpUI1',
    129:'__vbaFreeObj',
    130:'__vbaFreeStr',
    131:'__vbaFreeVar',
    132:'__vbaFreeVarg',
    133:'__vbaI2Abs',
    134:'__vbaI2I4',
    135:'__vbaI2Sgn',
    136:'__vbaI4Abs',
    137:'__vbaI4Sgn',
    138:'__vbaStrCopy',
    139:'__vbaStrMove',
    140:'__vbaUI1I2',
    141:'__vbaUI1I4',
    142:'__vbaUI1Sgn',
    143:'__vbaVarCopy',
    144:'__vbaVarDup',
    145:'__vbaVarMove',
    146:'__vbaVarVargNofree',
    147:'__vbaVarZero',
    148:'__vbaVargParmRef',
    149:'__vbaVargVar',
    150:'__vbaVargVarCopy',
    151:'__vbaVargVarMove',
    152:'__vbaVargVarRef',
    153:'DLLGetDocumentation',
    154:'DllCanUnloadNow',
    155:'DllGetClassObject',
    156:'_CIatan',
    157:'_CIcos',
    158:'_CIexp',
    159:'_CIlog',
    160:'_CIsin',
    161:'_CIsqrt',
    162:'_CItan',
    163:'__vbaAptOffset',
    164:'__vbaAryConstruct2',
    165:'__vbaAryConstruct',
    166:'__vbaAryCopy',
    167:'__vbaAryDestruct',
    168:'__vbaAryMove',
    169:'__vbaAryRebase1Var',
    170:'__vbaAryRecCopy',
    171:'__vbaAryRecMove',
    172:'__vbaAryUnlock',
    173:'__vbaAryVar',
    174:'__vbaBoolStr',
    175:'__vbaBoolVar',
    176:'__vbaBoolVarNull',
    177:'__vbaCVarAryUdt',
    178:'__vbaCastObj',
    179:'__vbaCastObjVar',
    180:'__vbaCheckType',
    181:'__vbaCheckTypeVar',
    182:'__vbaChkstk',
    183:'__vbaCopyBytes',
    184:'__vbaCopyBytesZero',
    185:'__vbaCyAbs',
    186:'ProcCallEngine',
    187:'DllFunctionCall',
    188:'__vbaCyAdd',
    189:'__vbaCyErrVar',
    190:'CopyRecord',
    191:'__vbaCyFix',
    192:'__vbaCyForInit',
    193:'__vbaCyForNext',
    194:'__vbaCyI2',
    195:'TipGetAddressOfPredeclaredInstance',
    196:'__vbaCyI4',
    197:'__vbaCyInt',
    198:'__vbaCyMul',
    199:'MethCallEngine',
    200:'__vbaCyMulI2',
    201:'__vbaCySgn',
    202:'__vbaCyStr',
    203:'__vbaCySub',
    204:'__vbaCyUI1',
    205:'__vbaCyVar',
    206:'__vbaDateR4',
    207:'__vbaDateR8',
    208:'__vbaDateStr',
    209:'__vbaDateVar',
    210:'__vbaDerefAry',
    211:'__vbaDerefAry1',
    212:'__vbaEnd',
    213:'__vbaErase',
    214:'__vbaEraseKeepData',
    215:'__vbaEraseNoPop',
    216:'__vbaError',
    217:'__vbaErrorOverflow',
    218:'__vbaExceptHandler',
    219:'__vbaExitEachAry',
    220:'TipSetOption',
    221:'__vbaExitEachColl',
    222:'__vbaExitEachVar',
    223:'__vbaExitProc',
    224:'__vbaFPException',
    225:'__vbaFPFix',
    226:'__vbaFPInt',
    227:'TipUnloadProject',
    228:'__vbaFailedFriend',
    229:'__vbaFileClose',
    230:'TipCreateInstanceProject2',
    231:'EbResetProject',
    232:'EbGetHandleOfExecutingProject',
    233:'__vbaFileCloseAll',
    234:'__vbaFileLock',
    235:'__vbaFileOpen',
    236:'__vbaFileSeek',
    237:'__vbaFixstrConstruct',
    238:'__vbaForEachAry',
    239:'__vbaForEachCollAd',
    240:'__vbaForEachCollObj',
    241:'__vbaForEachCollVar',
    242:'__vbaForEachVar',
    243:'__vbaFreeObjList',
    244:'__vbaFreeStrList',
    245:'__vbaFreeVarList',
    246:'__vbaGenerateBoundsError',
    247:'__vbaGet3',
    248:'__vbaGet4',
    249:'__vbaGetFxStr3',
    250:'__vbaGetFxStr4',
    251:'__vbaGetOwner3',
    252:'__vbaGetOwner4',
    253:'__vbaGosub',
    254:'__vbaGosubFree',
    255:'__vbaGosubReturn',
    256:'__vbaHresultCheck',
    257:'__vbaHresultCheckNonvirt',
    258:'__vbaHresultCheckObj',
    259:'__vbaI2Cy',
    260:'__vbaI2ErrVar',
    261:'__vbaI2ForNextCheck',
    262:'__vbaI2Str',
    263:'__vbaI2Var',
    264:'__vbaI4Cy',
    265:'EbResetProjectNormal',
    266:'TipUnloadInstance',
    267:'__vbaI4ErrVar',
    268:'EbLibraryLoad',
    269:'EbLibraryUnload',
    270:'__vbaI4ForNextCheck',
    271:'EbLoadRunTime',
    272:'__vbaI4Str',
    273:'__vbaI4Var',
    274:'EbCreateContext',
    275:'EbDestroyContext',
    276:'EbSetContextWorkerThread',
    277:'__vbaInStr',
    278:'__vbaInStrB',
    279:'__vbaInStrVar',
    280:'__vbaInStrVarB',
    281:'__vbaInputFile',
    282:'__vbaLateIdCall',
    283:'__vbaLateIdCallLd',
    284:'EbGetErrorInfo',
    285:'__vbaLateIdCallSt',
    286:'__vbaLateIdNamedCall',
    287:'__vbaLateIdNamedCallLd',
    288:'__vbaLateIdNamedCallSt',
    289:'__vbaLateIdNamedStAd',
    290:'__vbaLateIdSt',
    291:'__vbaLateIdStAd',
    292:'__vbaLateMemCall',
    293:'__vbaLateMemCallLd',
    294:'__vbaLateMemCallSt',
    295:'__vbaLateMemNamedCall',
    296:'__vbaLateMemNamedCallLd',
    297:'__vbaLateMemNamedCallSt',
    298:'EbIsProjectOnStack',
    299:'TipCreateInstanceEx',
    300:'GetMem2',
    301:'GetMem4',
    302:'GetMem8',
    303:'GetMemStr',
    304:'GetMemVar',
    305:'GetMemObj',
    306:'PutMem2',
    307:'PutMem4',
    308:'PutMem8',
    309:'PutMemStr',
    310:'PutMemVar',
    311:'PutMemObj',
    312:'SetMemVar',
    313:'SetMemObj',
    314:'GetMemNewObj',
    315:'PutMemNewObj',
    316:'SetMemNewObj',
    317:'GetMem1',
    318:'PutMem1',
    319:'GetMemEvent',
    320:'PutMemEvent',
    321:'SetMemEvent',
    322:'__vbaLateMemNamedStAd',
    323:'__vbaLateMemSt',
    324:'__vbaLateMemStAd',
    325:'__vbaLbound',
    326:'__vbaLdZeroAry',
    327:'__vbaLenBstr',
    328:'__vbaLenBstrB',
    329:'__vbaLenVar',
    330:'__vbaLenVarB',
    331:'__vbaLineInputStr',
    332:'__vbaLineInputVar',
    333:'__vbaLsetFixstr',
    334:'__vbaLsetFixstrFree',
    335:'__vbaMidStmtBstr',
    336:'__vbaMidStmtBstrB',
    337:'__vbaMidStmtVar',
    338:'__vbaMidStmtVarB',
    339:'__vbaNameFile',
    340:'__vbaNew2',
    341:'__vbaNew',
    342:'__vbaNextEachAry',
    343:'__vbaNextEachCollAd',
    344:'__vbaNextEachCollObj',
    345:'__vbaNextEachCollVar',
    346:'__vbaNextEachVar',
    347:'__vbaObjAddref',
    348:'__vbaObjIs',
    349:'__vbaObjSet',
    350:'__vbaObjSetAddref',
    351:'__vbaObjVar',
    352:'__vbaOnError',
    353:'__vbaOnGoCheck',
    354:'__vbaPowerR8',
    355:'__vbaPrintFile',
    356:'__vbaPrintObj',
    357:'__vbaPut3',
    358:'__vbaPut4',
    359:'__vbaPutFxStr3',
    360:'__vbaPutFxStr4',
    361:'__vbaPutOwner3',
    362:'__vbaPutOwner4',
    363:'__vbaR4Cy',
    364:'__vbaR4ErrVar',
    365:'__vbaR4ForNextCheck',
    366:'__vbaR4Sgn',
    367:'__vbaR4Str',
    368:'__vbaR4Var',
    369:'__vbaR8Cy',
    370:'__vbaR8ErrVar',
    371:'__vbaR8FixI2',
    372:'__vbaR8FixI4',
    373:'__vbaR8ForNextCheck',
    374:'__vbaR8IntI2',
    375:'__vbaR8IntI4',
    376:'__vbaR8Sgn',
    377:'__vbaR8Str',
    378:'__vbaR8Var',
    379:'__vbaRaiseEvent',
    380:'__vbaRecAnsiToUni',
    381:'__vbaRecAssign',
    382:'__vbaRecDestruct',
    383:'__vbaRecDestructAnsi',
    384:'__vbaRecUniToAnsi',
    385:'__vbaRedim',
    386:'__vbaRedimPreserve',
    387:'__vbaRedimPreserveVar',
    388:'__vbaRedimPreserveVar2',
    389:'__vbaRedimVar',
    390:'__vbaRefVarAry',
    391:'__vbaResume',
    392:'__vbaRsetFixstr',
    393:'__vbaRsetFixstrFree',
    394:'__vbaSetSystemError',
    395:'__vbaStopExe',
    396:'__vbaStr2Vec',
    397:'__vbaStrAryToAnsi',
    398:'__vbaStrAryToUnicode',
    399:'__vbaStrBool',
    400:'EVENT_SINK_QueryInterface',
    401:'EVENT_SINK_AddRef',
    402:'EVENT_SINK_Release',
    403:'EVENT_SINK_GetIDsOfNames',
    404:'EVENT_SINK_Invoke',
    405:'__vbaStrCat',
    406:'__vbaStrCmp',
    407:'__vbaStrComp',
    408:'__vbaStrCompVar',
    409:'__vbaStrCy',
    410:'BASIC_CLASS_QueryInterface',
    411:'BASIC_CLASS_AddRef',
    412:'BASIC_CLASS_Release',
    413:'BASIC_CLASS_GetIDsOfNames',
    414:'BASIC_CLASS_Invoke',
    415:'__vbaStrDate',
    416:'__vbaStrFixstr',
    417:'__vbaStrI2',
    418:'__vbaStrI4',
    419:'__vbaStrLike',
    420:'BASIC_DISPINTERFACE_GetTICount',
    421:'BASIC_DISPINTERFACE_GetTypeInfo',
    422:'__vbaStrR4',
    423:'__vbaStrR8',
    424:'__vbaStrTextCmp',
    425:'__vbaStrTextLike',
    426:'__vbaStrToAnsi',
    427:'__vbaStrToUnicode',
    428:'__vbaStrUI1',
    429:'__vbaStrVarCopy',
    430:'Zombie_QueryInterface',
    431:'Zombie_AddRef',
    432:'Zombie_Release',
    433:'Zombie_GetTypeInfoCount',
    434:'Zombie_GetTypeInfo',
    435:'Zombie_GetIDsOfNames',
    436:'Zombie_Invoke',
    437:'__vbaStrVarMove',
    438:'__vbaStrVarVal',
    439:'__vbaUI1Cy',
    440:'EVENT_SINK2_AddRef',
    441:'EVENT_SINK2_Release',
    442:'__vbaUI1ErrVar',
    443:'__vbaUI1Str',
    444:'__vbaUI1Var',
    445:'__vbaUbound',
    446:'__vbaUdtVar',
    447:'__vbaUnkVar',
    448:'__vbaVar2Vec',
    449:'__vbaVarAbs',
    450:'__vbaVarAdd',
    451:'__vbaVarAnd',
    452:'__vbaVarCat',
    453:'__vbaVarCmpEq',
    454:'__vbaVarCmpGe',
    455:'__vbaVarCmpGt',
    456:'__vbaVarCmpLe',
    457:'__vbaVarCmpLt',
    458:'__vbaVarCmpNe',
    459:'__vbaVarDateVar',
    460:'__vbaVarDiv',
    461:'__vbaVarEqv',
    462:'__vbaVarErrI4',
    463:'__vbaVarFix',
    464:'__vbaVarForInit',
    465:'__vbaVarForNext',
    466:'__vbaVarIdiv',
    467:'__vbaVarImp',
    468:'__vbaVarIndexLoad',
    469:'__vbaVarIndexLoadRef',
    470:'__vbaVarIndexLoadRefLock',
    471:'__vbaVarIndexStore',
    472:'__vbaVarIndexStoreObj',
    473:'__vbaVarInt',
    474:'__vbaVarLike',
    475:'__vbaVarLikeVar',
    476:'__vbaVarMod',
    477:'__vbaVarMul',
    478:'__vbaVarNeg',
    479:'__vbaVarNot',
    480:'__vbaVarOr',
    481:'__vbaVarPow',
    482:'__vbaVarSetObj',
    483:'__vbaVarSetObjAddref',
    484:'__vbaVarSetUnk',
    485:'__vbaVarSetUnkAddref',
    486:'__vbaVarSetVar',
    487:'__vbaVarSetVarAddref',
    488:'__vbaVarSub',
    489:'__vbaVarTextCmpEq',
    490:'__vbaVarTextCmpGe',
    491:'__vbaVarTextCmpGt',
    492:'__vbaVarTextCmpLe',
    493:'__vbaVarTextCmpLt',
    494:'__vbaVarTextCmpNe',
    495:'__vbaVarTextLike',
    496:'__vbaVarTextLikeVar',
    497:'__vbaVarTextTstEq',
    498:'__vbaVarTextTstGe',
    499:'__vbaVarTextTstGt',
    500:'__vbaVarTextTstLe',
    501:'__vbaVarTextTstLt',
    502:'__vbaVarTextTstNe',
    503:'__vbaVarTstEq',
    504:'__vbaVarTstGe',
    505:'__vbaVarTstGt',
    506:'__vbaVarTstLe',
    507:'__vbaVarTstLt',
    508:'__vbaVarTstNe',
    509:'__vbaVarXor',
    510:'__vbaVargObj',
    511:'__vbaVargObjAddref',
    512:'rtcLeftBstr',
    513:'rtcLeftVar',
    514:'rtcRightBstr',
    515:'rtcRightVar',
    516:'rtcAnsiValueBstr',
    517:'rtcLowerCaseBstr',
    518:'rtcLowerCaseVar',
    519:'rtcTrimBstr',
    520:'rtcTrimVar',
    521:'rtcLeftTrimBstr',
    522:'rtcLeftTrimVar',
    523:'rtcRightTrimBstr',
    524:'rtcRightTrimVar',
    525:'rtcSpaceBstr',
    526:'rtcSpaceVar',
    527:'rtcUpperCaseBstr',
    528:'rtcUpperCaseVar',
    529:'rtcKillFiles',
    530:'rtcChangeDir',
    531:'rtcMakeDir',
    532:'rtcRemoveDir',
    533:'rtcChangeDrive',
    534:'rtcBeep',
    535:'rtcGetTimer',
    536:'rtcStrFromVar',
    537:'rtcBstrFromAnsi',
    538:'rtcPackDate',
    539:'rtcPackTime',
    540:'rtcGetDateValue',
    541:'rtcGetTimeValue',
    542:'rtcGetDayOfMonth',
    543:'rtcGetHourOfDay',
    544:'rtcGetMinuteOfHour',
    545:'rtcGetMonthOfYear',
    546:'rtcGetPresentDate',
    547:'rtcGetSecondOfMinute',
    548:'rtcSetDateVar',
    549:'rtcSetDateBstr',
    550:'rtcSetTimeVar',
    551:'rtcSetTimeBstr',
    552:'rtcGetDayOfWeek',
    553:'rtcGetYear',
    554:'rtcFileReset',
    555:'rtcFileAttributes',
    556:'rtcIsArray',
    557:'rtcIsDate',
    558:'rtcIsEmpty',
    559:'rtcIsError',
    560:'rtcIsNull',
    561:'rtcIsNumeric',
    562:'rtcIsObject',
    563:'rtcVarType',
    564:'rtDecFromVar',
    565:'rtcFileWidth',
    566:'rtcInputCount',
    567:'rtcInputCountVar',
    568:'rtcFileSeek',
    569:'rtcFileLocation',
    570:'rtcFileLength',
    571:'rtcEndOfFile',
    572:'rtcHexBstrFromVar',
    573:'rtcHexVarFromVar',
    574:'rtcOctBstrFromVar',
    575:'rtcOctVarFromVar',
    576:'rtcFileCopy',
    577:'rtcFileDateTime',
    578:'rtcFileLen',
    579:'rtcGetFileAttr',
    580:'rtcSetFileAttr',
    581:'rtcR8ValFromBstr',
    582:'rtcSin',
    583:'rtcCos',
    584:'rtcTan',
    585:'rtcAtn',
    586:'rtcExp',
    587:'rtcLog',
    588:'rtcRgb',
    589:'rtcQBColor',
    590:'rtcMacId',
    591:'rtcTypeName',
    592:'rtcIsMissing',
    593:'rtcRandomNext',
    594:'rtcRandomize',
    595:'rtcMsgBox',
    596:'rtcInputBox',
    597:'rtcAppActivate',
    598:'rtcDoEvents',
    599:'rtcSendKeys',
    600:'rtcShell',
    601:'rtcArray',
    602:'__vbaVargUnk',
    603:'__vbaVargUnkAddref',
    604:'__vbaVerifyVarObj',
    605:'rtcGetErl',
    606:'rtcStringBstr',
    607:'rtcStringVar',
    608:'rtcVarBstrFromAnsi',
    609:'rtcGetDateBstr',
    610:'rtcGetDateVar',
    611:'rtcGetTimeBstr',
    612:'rtcGetTimeVar',
    613:'rtcVarStrFromVar',
    614:'rtcSqr',
    615:'rtcIMEStatus',
    616:'rtcLeftCharBstr',
    617:'rtcLeftCharVar',
    618:'rtcRightCharBstr',
    619:'rtcRightCharVar',
    620:'rtcInputCharCount',
    621:'rtcInputCharCountVar',
    622:'rtcStrConvVar',
    623:'__vbaWriteFile',
    624:'rtcGetHostLCID',
    625:'rtcCreateObject',
    626:'rtcGetObject',
    627:'rtcAppleScript',
    628:'rtcMidBstr',
    629:'rtcMidVar',
    630:'rtcInStr',
    631:'rtcMidCharBstr',
    632:'rtcMidCharVar',
    633:'rtcInStrChar',
    634:'rtBstrFromErrVar',
    635:'rtBoolFromErrVar',
    636:'rtCyFromErrVar',
    637:'rtI2FromErrVar',
    638:'rtI4FromErrVar',
    639:'rtR4FromErrVar',
    640:'rtR8FromErrVar',
    641:'rtcDateFromVar',
    642:'rtcVarFromVar',
    643:'rtcCVErrFromVar',
    644:'VarPtr',
    645:'rtcDir',
    646:'rtcCurrentDirBstr',
    647:'rtcCurrentDir',
    648:'rtcFreeFile',
    649:'rtcCompareBstr',
    650:'rtcBstrFromFormatVar',
    651:'rtcBstrFromError',
    652:'rtcVarFromError',
    653:'rtcLenCharVar',
    654:'rtcLenVar',
    655:'rtcFixVar',
    656:'rtcAbsVar',
    657:'rtcIntVar',
    658:'rtcSgnVar',
    659:'_adj_fdiv_m16i',
    660:'rtcVarFromFormatVar',
    661:'rtcDateAdd',
    662:'rtcDateDiff',
    663:'rtcDatePart',
    664:'rtcPartition',
    665:'rtcChoose',
    666:'rtcEnvironVar',
    667:'rtcEnvironBstr',
    668:'rtcSwitch',
    669:'rtcCommandBstr',
    670:'rtcCommandVar',
    671:'rtcSLN',
    672:'rtcSYD',
    673:'rtcDDB',
    674:'rtcIPMT',
    675:'rtcPPMT',
    676:'rtcPMT',
    677:'rtcPV',
    678:'rtcFV',
    679:'rtcNPer',
    680:'rtcRate',
    681:'rtcImmediateIf',
    682:'rtcIRR',
    683:'rtcMIRR',
    684:'rtcNPV',
    685:'rtcErrObj',
    686:'rtUI1FromErrVar',
    687:'rtcVarDateFromVar',
    688:'_adj_fdiv_m32',
    689:'rtcGetSetting',
    690:'rtcSaveSetting',
    691:'rtcDeleteSetting',
    692:'rtcGetAllSettings',
    693:'rtcByteValueBstr',
    694:'rtcBstrFromByte',
    695:'rtcVarBstrFromByte',
    696:'rtcCharValueBstr',
    697:'rtcBstrFromChar',
    698:'rtcVarBstrFromChar',
    699:'rtcSetCurrentCalendar',
    700:'rtcGetCurrentCalendar',
    701:'_adj_fdiv_m32i',
    702:'rtcFormatNumber',
    703:'rtcFormatCurrency',
    704:'rtcFormatPercent',
    705:'rtcFormatDateTime',
    706:'rtcWeekdayName',
    707:'rtcMonthName',
    708:'rtcFilter',
    709:'rtcInStrRev',
    710:'rtcJoin',
    711:'rtcSplit',
    712:'rtcReplace',
    713:'rtcStrReverse',
    714:'rtcRound',
    715:'rtcCallByName',
    716:'rtcCreateObject2',
    717:'rtcStrConvVar2',
    718:'_adj_fdiv_m64',
    719:'_adj_fdiv_r',
    720:'_adj_fdivr_m16i',
    721:'_adj_fdivr_m32',
    722:'_adj_fdivr_m32i',
    723:'_adj_fdivr_m64',
    724:'_adj_fpatan',
    725:'_adj_fprem',
    726:'_adj_fprem1',
    727:'_adj_fptan',
    728:'_allmul',
    999:'TipInvokeMethod2',
    1016:'TipInvokeMethod',
    1024:'IID_IVbaHost',
    1025:'EbGetObjConnectionCounts',
    2000:'CreateIExprSrvObj',
    2010:'EbGetVBAObject',
}



import logging
import unittest

import envi

import vivisect
import vivisect.const as v_const

import vivisect.tests.helpers as helpers
import vivisect.tests.utils as v_t_utils

class LinuxTests(v_t_utils.VivTest):

    def test_linux_x86_syscall(self):
        vw = helpers.getTestWorkspace('linux', 'i386', 'syscalls.elf')
        # address of the main function
        fva = 0x02001190
        self.eq(fva, vw.getFunction(fva))
        cb = vw.getCodeBlock(fva)
        loc = vw.getLocation(cb[0] + cb[1] - 1)
        self.eq(loc[0], 0x20011ab)
        self.eq(loc[1], 2)
        self.eq(loc[2], v_const.LOC_OP)
        self.eq(loc[3], envi.ARCH_I386 | envi.IF_PRIV)
        op = vw.parseOpcode(loc[0])
        self.eq(repr(op), 'int 128')
        self.eq(True, vw.isNoReturnVa(loc[0]))



'''
Some utilities for dealing with COFF .LIB files
'''

import binascii

import vstruct
from vstruct.defs.pe import *
from vstruct.primitives import *

IMAGE_ARCHIVE_START_SIZE             = 8
IMAGE_ARCHIVE_START                  = '!<arch>\n'
IMAGE_ARCHIVE_END                    = '`\n'
IMAGE_ARCHIVE_PAD                    = '\n'
IMAGE_ARCHIVE_LINKER_MEMBER          = '/               '
IMAGE_ARCHIVE_LONGNAMES_MEMBER       = '//              '

IMAGE_ARCHIVE_HEADER_SIZE           = 60

class IMAGE_ARCHIVE_MEMBER_HEADER(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Name       = v_str(size=16)
        self.Date       = v_str(size=12)
        self.UserID     = v_str(size=6)
        self.GroupID    = v_str(size=6)
        self.Mode       = v_str(size=8)
        self.Size       = v_str(size=10)
        self.EndHeader  = v_str(size=2)
        self.FileData   = vstruct.VStruct()

class IMAGE_ARCHIVE_MEMBER(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.MemberHeader = IMAGE_ARCHIVE_MEMBER_HEADER()

    def vsParse(self, bytes, offset=0):
        off = vstruct.VStruct.vsParse(self, bytes, offset=offset)

        # Use the size that the header says we are...
        offset = offset + len(self.MemberHeader) + int( self.MemberHeader.Size )

        # two byte aligned
        if offset % 2:
            offset += 1

        return offset

class IMAGE_ARCHIVE_LINKER1(IMAGE_ARCHIVE_MEMBER):

    def __init__(self):
        IMAGE_ARCHIVE_MEMBER.__init__(self)
        self.NumberOfSymbols = v_uint32(bigend=True)
        self.SymbolOffsets = vstruct.VArray()
        self.SymbolNames = vstruct.VArray()

    def pcb_NumberOfSymbols(self):
        c = self.NumberOfSymbols
        self.SymbolOffsets = vstruct.VArray( elems=[ v_uint32(bigend=True) for i in range(c) ])
        self.SymbolNames = vstruct.VArray( elems=[ v_zstr() for i in range(c) ])

class IMAGE_ARCHIVE_LINKER2(IMAGE_ARCHIVE_MEMBER):

    def __init__(self):
        IMAGE_ARCHIVE_MEMBER.__init__(self)
        self.NumberOfMembers = v_uint32()
        self.MemberOffsets = vstruct.VArray()
        self.NumberOfSymbols = v_uint32()
        self.SymbolIndexes = vstruct.VArray()
        self.SymbolNames = vstruct.VArray()

    def pcb_NumberOfMembers(self):
        c = self.NumberOfMembers
        self.MemberOffsets = vstruct.VArray( elems=[ v_uint32() for i in range(c) ] )

    def pcb_NumberOfSymbols(self):
        c = self.NumberOfSymbols
        self.SymbolIndexes = vstruct.VArray( elems=[ v_uint16() for i in range(c) ])
        self.SymbolNames = vstruct.VArray( elems=[ v_zstr() for i in range(c) ])

IMPORT_SIG      = binascii.unhexlify('0000ffff')
IMPORT_CODE     = 0 # Executable code.
IMPORT_DATA     = 1 # Data.
IMPORT_CONST    = 2 # Specified as CONST in the .def file.

class IMAGE_ARCHIVE_IMPORT(IMAGE_ARCHIVE_MEMBER):

    def __init__(self):
        IMAGE_ARCHIVE_MEMBER.__init__(self)
        self.Sig1           = v_uint16()
        self.Sig2           = v_uint16()
        self.Version        = v_uint16()
        self.Machine        = v_uint16()
        self.DateTimeStamp  = v_uint32()
        self.SizeOfData     = v_uint32()
        self.Ordinal        = v_uint16()
        self.ImportFlags    = v_uint16()
        self.ImportName     = v_zstr()
        self.ImportLibName  = v_zstr()

class IMAGE_COFF_SYMBOL(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)

        self.u                  = vstruct.VUnion()
        self.u.Name             = v_str(size=8)
        self.u.lname            = vstruct.VStruct()
        self.u.lname.Zeros      = v_uint32()
        self.u.lname.Offset     = v_uint32()

        self.Value              = v_uint32()
        self.SectionNumber      = v_uint16()
        self.Type               = v_uint16()
        self.StorageClass       = v_uint8()
        self.NumberOfAuxSymbols = v_uint8()
        self.AuxSymbols         = vstruct.VArray()

    def pcb_NumberOfAuxSymbols(self):
        a = self.NumberOfAuxSymbols
        self.AuxSymbols = vstruct.VArray(elems=[ v_bytes(size=18) for i in range(a) ])

class IMAGE_ARCHIVE_COFF(IMAGE_ARCHIVE_MEMBER):

    def __init__(self, bigend=False):
        IMAGE_ARCHIVE_MEMBER.__init__(self)
        self.FileHeader = IMAGE_FILE_HEADER()
        self.SectionHeaders = vstruct.VArray()
        self.SectionData = v_bytes()
        self.SymbolTable = vstruct.VArray()

    def pcb_FileHeader(self):
        c = self.FileHeader.NumberOfSections
        self.SectionHeaders = vstruct.VArray(elems=[ IMAGE_SECTION_HEADER() for i in range(c) ])
        p = self.FileHeader.PointerToSymbolTable
        if p != 0:
            s = self.FileHeader.NumberOfSymbols
            p -= len(self.FileHeader)
            p -= len(self.SectionHeaders)
            self.vsGetField('SectionData').vsSetLength(p)
            self.SymbolTable = vstruct.VArray(elems=[IMAGE_COFF_SYMBOL() for i in range(s)])

class IMAGE_ARCHIVE(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.Magic = v_bytes(vbytes=IMAGE_ARCHIVE_START)
        # A set of image archive names is first...
        self.ImageArchiveMembers = vstruct.VArray(elems=(IMAGE_ARCHIVE_LINKER1(), IMAGE_ARCHIVE_LINKER2()))
        #self.ImageArchiveMembers = vstruct.VArray()

    def vsParse(self, bytes, offset=0):
        blen = len(bytes)
        offset = self.vsGetField('Magic').vsParse(bytes, offset=offset)
        # Parse the "names" headers
        offset = self.vsGetField('ImageArchiveMembers').vsParse(bytes, offset=offset)

        while offset < blen:

            doff = offset + IMAGE_ARCHIVE_HEADER_SIZE

            if bytes.startswith(IMPORT_SIG, doff):
                memb = IMAGE_ARCHIVE_IMPORT()

            elif bytes.startswith('// ', offset):
                # Skip the depricated lib strings crap...
                memb = IMAGE_ARCHIVE_MEMBER()

            else:
                memb = IMAGE_ARCHIVE_COFF()

            offset = memb.vsParse(bytes, offset=offset)
            self.ImageArchiveMembers.vsAddElement(memb)

        return offset



"""
A file full of bit twidling helpers
"""

import struct

MAX_WORD = 32  # usually no more than 8, 16 is for SIMD register support

# Masks to use for unsigned anding to size
u_maxes = [(2 ** (8*i)) - 1 for i in range(MAX_WORD+1)]
u_maxes[0] = 0  # powers of 0 are 1, but we need 0
bu_maxes = [(2 ** (i)) - 1 for i in range(8*MAX_WORD+1)]

# Masks of just the sign bit for different sizes
sign_bits = [(2 ** (8*i)) >> 1 for i in range(MAX_WORD+1)]
sign_bits[0] = 0  # powers of 0 are 1, but we need 0
bsign_bits = [(2 ** i) >> 1 for i in range(8*MAX_WORD+1)]

# Max *signed* masks (all but top bit )
s_maxes = [u_maxes[i] ^ sign_bits[i] for i in range(len(u_maxes))]
s_maxes[0] = 0

# bit width masks
b_masks = [(2**i)-1 for i in range(MAX_WORD*8)]
b_masks[0] = 0

def unsigned(value, size):
    """
    Make a value unsigned based on it's size.
    """
    return value & u_maxes[size]

def signed(value, size):
    """
    Make a value signed based on it's size.
    """
    x = unsigned(value, size)
    if x & sign_bits[size]:
        x = (x - u_maxes[size]) - 1
    return x

def bsigned(value, size):
    """
    Make a value signed based on it's size.
    """
    if value & bsign_bits[size]:
        value = (value - bu_maxes[size]) - 1
    return value

def is_signed(value, size):
    x = unsigned(value, size)
    return bool(x & sign_bits[size])

def sign_extend(value, cursize, newsize):
    """
    Take a value and extend it's size filling
    in the space with the value of the high
    order bit.
    """
    x = unsigned(value, cursize)
    if cursize != newsize:
        # Test for signed w/o the call
        if x & sign_bits[cursize]:
            delta = newsize - cursize
            highbits = u_maxes[delta]
            x |= highbits << (8*cursize)
    return x

def bsign_extend(value, cursize, newsize):
    x = value
    if cursize != newsize:
        if x & bsign_bits[cursize]:
            delta = newsize - cursize
            highbits = bu_maxes[delta]
            x |= highbits << (cursize)
    return x

def is_parity(val):
    s = 0
    while val:
        s ^= val & 1
        val = val >> 1
    return (not s)

parity_table = []
for i in range(256):
    parity_table.append(is_parity(i))

def is_parity_byte(bval):
    """
    An "optimized" parity checker that looks up the index.
    """
    return parity_table[bval & 0xff]

def lsb(value):
    return value & 0x1

def msb(value, size):
    return bool(value & sign_bits[size])

def msb_minus_one(value, size):
    bsize = size << 3
    return bool(value & bsign_bits[bsize-1])

def is_signed_half_carry(value, size, src):
    '''
    BCD carry/borrow in the second most important nibble:
        32bit   - bit 27
        16bit   - bit 11
        8bit    - bit 3
    '''
    bitsize = (size << 3) - 5
    mask = 1<<bitsize

    p1 = value & mask
    p2 = src & mask

    return ((p1 ^ p2) != 0)

def is_signed_carry(value, size, src):
    smax = s_maxes[size]
    if value > smax > src:
        return True
    if value < -smax < -src:
        return True
    return False

def is_signed_overflow(value, size):
    smax = s_maxes[size]
    if value > smax:
        return True
    if value < -smax:
        return True
    return False

def is_unsigned_carry(value, size):
    umax = u_maxes[size]
    if value > umax:
        return True
    elif value < 0:
        return True
    return False

def is_aux_carry(src, dst):
    return (dst & 0xf) + (src & 0xf) > 15

def is_aux_carry_sub(src, dst):
    return src & 0xf > dst & 0xf

# set of format lists which make size, endianness, and signedness fast and easy
le_fmt_chars = (None, "B", "<H", None, "<I", None, None, None, "<Q")
be_fmt_chars = (None, "B", ">H", None, ">I", None, None, None, ">Q")
fmt_chars = (le_fmt_chars, be_fmt_chars)

le_fmt_schars = (None,"b","<h",None,"<i",None,None,None,"<q")
be_fmt_schars = (None,"b",">h",None,">i",None,None,None,">q")
fmt_schars = (le_fmt_schars, be_fmt_schars)

master_fmts = (fmt_chars, fmt_schars)

fmt_sizes =  (None,1,2,4,4,8,8,8,8)

le_fmt_float = (None, None, None, None, '<f', None, None, None, '<d')
be_fmt_float = (None, None, None, None, '>f', None, None, None, '>d')

fmt_floats = (le_fmt_float, be_fmt_float)


def getFormat(size, big_endian=False, signed=False):
    '''
    Returns the proper struct format for numbers up to 8 bytes in length
    Endianness and Signedness aware.

    Only useful for *full individual* numbers... ie. 1, 2, 4, 8.  Numbers
    of 24-bits (3), 40-bit (5), 48-bits (6) or 56-bits (7) are not accounted
    for here and will return None.
    '''
    return master_fmts[signed][big_endian][size]

def getFloatFormat(size, big_endian=False):
    '''
    Returns the proper struct format for numbers up to 8 bytes in length
    Endianness and Signedness aware.

    Only useful for *full individual* numbers... ie. 1, 2, 4, 8.  Numbers
    of 24-bits (3), 40-bit (5), 48-bits (6) or 56-bits (7) are not accounted
    for here and will return None.
    '''
    return fmt_floats[big_endian][size]

def parsebytes(bytes, offset, size, sign=False, bigend=False):
    """
    Mostly for pulling immediates out of strings...
    """
    if size > 8:
        return slowparsebytes(bytes, offset, size, sign=sign, bigend=bigend)
    if bigend:
        f = be_fmt_chars[size]
    else:
        f = le_fmt_chars[size]
    if f is None:
        return slowparsebytes(bytes, offset, size, sign=sign, bigend=bigend)
    d = bytes[offset:offset+size]
    x = struct.unpack(f, d)[0]
    if sign:
        x = signed(x, size)
    return x

def slowparsebytes(bytes, offset, size, sign=False, bigend=False):
    if bigend:
        begin = offset
        inc = 1
    else:
        begin = offset + (size-1)
        inc = -1

    ret = 0
    ioff = 0
    for x in range(size):
        ret = ret << 8
        ret |= bytes[begin+ioff]
        ioff += inc
    if sign:
        ret = signed(ret, size)
    return ret

def buildbytes(value, size, bigend=False):
    value = unsigned(value, size)
    if bigend:
        f = be_fmt_chars[size]
    else:
        f = le_fmt_chars[size]
    if f is None:
        raise Exception("envi.bits.buildbytes needs slowbuildbytes")
    return struct.pack(f, value)

def byteswap(value, size):
    ret = 0
    for i in range(size):
        ret = ret << 8
        ret |= (value >> (8*i)) & 0xff
    return ret

hex_fmt = {
    0:'0x%.1x',
    1:"0x%.2x",
    2:"0x%.4x",
    4:"0x%.8x",
    8:"0x%.16x",
}

def intwidth(val):
    if val < 0:
        val = abs(val)
    ret = 0
    while val:
        ret += 1
        val = val >> 8
    return ret

def hex(value, size=None):
    if size is None:
        size = intwidth(value)

    fmt = hex_fmt.get(size)
    if fmt is not None:
        return fmt % value

    x = []
    while value:
        x.append('%.2x' % (value & 0xff))
        value = value >> 8
    x.reverse()
    return '0x%.s' % ''.join(x)


    return hex_fmt.get(size) % value

def binrepr(intval, bitwidth=None):
    '''
    Return a string of one's and zero's for the given value.
    '''
    ret = []
    while intval:
        ret.append(str(intval & 0x1))
        intval >>= 1
    ret.reverse()
    binstr = ''.join(ret)
    if bitwidth is not None:
        binstr = binstr.rjust(bitwidth, '0')
    return binstr

def binary(binstr):
    '''
    Decode a binary string of 1/0's into a python number
    '''
    return int(binstr, 2)

def binbytes(binstr):
    '''
    Decode a binary string of 1/0's into a python binary
    string.
    '''
    if len(binstr) % 8 != 0:
        raise Exception('Byte padded binary strings only for now!')
    bytez = ''
    while binstr:
        bytez += chr(binary(binstr[:8]))
        binstr = binstr[8:]
    return bytez

def parsebits(bytes, offset, bitoff, bitsize):
    '''
    Parse bitsize bits from the bit offset bitoff beginning
    at offset bytes.

    Example:
    '''
    val = 0
    cnt = 0
    while cnt < bitsize:

        addbit = bitoff + cnt
        addoff = offset + (addbit >> 3)

        modoff = addbit % 8

        o = bytes[addoff]
        val = (val << 1) + ((o >> (7 - modoff)) & 1)

        cnt += 1

    return val

def masktest(s):
    '''
    Specify a bit mask with the following syntax:
    '110100xxx00xx' to return a tester callback which will
    determine if an integer value meets the mask.

    example:
        opcode = 0x4388e234
        if masktest('1011xxxx0000')(opcode):
            print('MATCHED!')

    NOTE: For performance reasons, it is recommeneded that
    masktest be used to initialize a static list of tests
    that are re-used rather than reconstructed.
    '''
    maskin = binary(s.replace('0', '1').replace('x', '0'))
    matchval = binary(s.replace('x', '0'))

    def domask(testval):
        return testval & maskin == matchval
    return domask

def align(origsize, alignment):
    '''
    Returns an aligned size based on alignment argument
    '''
    remainder = origsize % alignment
    if remainder == 0:
        return origsize
    else:
        return origsize + (alignment - remainder)

